#!/bin/sh
#
# author: Joerg Heinemann
# website: http://enterprise-log-management-appliance.org
# license: GPL v3
#
# /usr/share/elma/lib/elma-global-functions.sh
#


# #########################################################################################
#
# Global functions
#
# #########################################################################################


# ######### Error exit #########
# 
error_exit(){

	echo 1>&2
	echo "Scriptname : $(basename $0)" 1>&2
	echo "Version    : $ELMA_VERSION" 1>&2
	echo "Library    : $(basename $BASH_SOURCE)" 1>&2
	echo "Path       : $(dirname $BASH_SOURCE)" 1>&2

	if [ -f $ELMA_INSTALL_LOG_FILE ]
	then
		echo "Log file   : $ELMA_INSTALL_LOG_FILE" 1>&2
	else
		echo "Log file   : NOT found" 1>&2
	fi

	echo 1>&2
	echo "ABORT on line ${1:-"Unknown"} - Please check above error messages and installation log file!" 1>&2
	echo 1>&2

	exit 1

}

# ######### Library version #########
#
library_version(){

	local RETVAL=0

	echo "--- `date +%T.%N | cut -c 1-12` - Reload global functions ------------------------------------------------"
	echo
	echo "Library: $(basename $BASH_SOURCE)"
	echo "Path   : $(dirname $BASH_SOURCE)/" 
	echo

	return $RETVAL

}

# ######### Script header #########
#
script_header(){

	local RETVAL=0

	local OPTIONS="$1"

	pkill -of /usr/lib/packagekitd

	HOST_NAME=$(hostname) ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	HOST_IP=$(hostname -i) ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	HOST_IP=$(ifconfig | grep -Eo 'inet (addr:)?([0-9]*\.){3}[0-9]*' | grep -Eo '([0-9]*\.){3}[0-9]*' | grep -v '127.0.0.1') ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	HOST_DOMAIN=$(hostname -d) ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	HOST_DOMAIN=$(cat /etc/HOSTNAME | cut -d'.' -f2-100) ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ -z "$(echo $HOST_NAME | grep "$HOST_DOMAIN")" ]
	then
		HOST_FQDN="$HOST_NAME.$HOST_DOMAIN" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		HOST_FQDN="$HOST_NAME" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	check_internet_access "/dev/null"

	# ######### Update ELMA version variables #########
	#
	check_elma_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	echo
	echo "==========================================================================================="
	echo "Scriptname     : $(basename $0) $OPTIONS"
	echo "-------------------------------------------------------------------------------------------"
	echo
	echo "Version        : $ELMA_VERSION"
	echo "Library        : $(basename $BASH_SOURCE)" 
	echo "Path           : $(dirname $BASH_SOURCE)"

	if [ -f $ELMA_INSTALL_LOG_FILE ]
	then
		echo "Log file       : $ELMA_INSTALL_LOG_FILE"
	else
		echo "Log file       : NOT found"
	fi
	echo

	if [ -z $HOST_NAME ]
	then
		echo "Hostname       : Network configuration ERROR" ; error_exit "$LINENO"
	else
		echo "Hostname       : $HOST_NAME"
	fi

	if [ -z $HOST_DOMAIN ]
	then
		echo "DNS-Suffix     : Network configuration ERROR" ; error_exit "$LINENO"
	else
		echo "DNS-Suffix     : $HOST_DOMAIN"
	fi

	if [ -z $HOST_FQDN ]
	then
		echo "FQDN           : Network configuration ERROR" ; error_exit "$LINENO"
	else
		echo "FQDN           : $HOST_FQDN"
	fi

	if [ -z $HOST_IP ]
	then
		echo "IP address     : Network configuration ERROR" ; error_exit "$LINENO"
	else
		echo "IP address     : $HOST_IP"
	fi

	if [ -z $INTERNET_ACCESS ]
	then
		echo "Internet-Access: Network configuration ERROR"
	else
		echo "Internet-Access: $INTERNET_ACCESS"
	fi

	echo
	echo "Description    : $SCRIPT_DESCRIPTION"
	echo
	echo "Author         : $SCRIPT_AUTHOR"
	echo "Website        : $PROJECT_HOME"
	echo
	echo "==========================================================================================="
	echo
	return $RETVAL

}

# ######### Logging setup #########
# 
script_logging(){

	local RETVAL=0
	LOGGING=1

	ELMA_LOG_DIR=/var/log/elma
	ELMA_INSTALL_LOG_FILE=$ELMA_LOG_DIR/$(basename $0 | awk -F'.' '{print $1}')-`date '+%Y-%m-%d-%H:%M:%S'`.log
	ELMA_INSTALL_STDOUT_AND_STDERR="$ELMA_LOG_DIR/stdout-stderr"

	if [ ! -d $ELMA_LOG_DIR ]
	then
		mkdir $ELMA_LOG_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	mkfifo ${ELMA_INSTALL_LOG_FILE}.pipe ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	mkfifo ${ELMA_INSTALL_STDOUT_AND_STDERR}.pipe ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	while read data; do echo $data >> $ELMA_INSTALL_LOG_FILE; done < ${ELMA_INSTALL_LOG_FILE}.pipe &

	# Redirect stdout into named pipe running "tee"
	#
	tee < ${ELMA_INSTALL_STDOUT_AND_STDERR}.pipe ${ELMA_INSTALL_LOG_FILE}.pipe &
	exec &> ${ELMA_INSTALL_STDOUT_AND_STDERR}.pipe 4>${ELMA_INSTALL_LOG_FILE}.pipe

	rm ${ELMA_INSTALL_LOG_FILE}.pipe ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	rm ${ELMA_INSTALL_STDOUT_AND_STDERR}.pipe ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# Redirect file decriptor into named pipe"
	#
	# if [ $LOGGING -eq 1 ]; then echo "test" >&4; fi
	# if [ $LOGGING -eq 1 ]; then grep "test" * >&4; fi

	return $RETVAL

}

script_log_file(){

	local RETVAL=0

	echo "Installation log file:"
	echo "----------------------"
	echo

	if [ -f $ELMA_INSTALL_LOG_FILE ]
	then
		echo "$ELMA_INSTALL_LOG_FILE"
	fi

	return $RETVAL

}


# #########################################################################################
#
# openSUSE installation, patch and update packages functions
#
# #########################################################################################

# ######### Patch openSUSE packages #########
#
patch_opensuse_packages(){

	local RETVAL=0
	SUSE_UPDATE_OUTPUT=""

	# ######### Reconfigure system distribution repositories #########
	#
	reconfigure_update_repositories ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ ! -z $INTERNET_ACCESS ]
	then

		echo "--- `date +%T.%N | cut -c 1-12` - Patch openSUSE packages ------------------------------------------------"
		echo
		zypper -n --no-gpg-checks --gpg-auto-import-keys refresh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo

		SUSE_UPDATE_OUTPUT="$(zypper -n --no-gpg-checks --gpg-auto-import-keys patch)" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo "$SUSE_UPDATE_OUTPUT"
		echo

		# ######### Check differences between system and distribution config files #########
		#
		check_config_file_diffs ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		# ######### Reload ELMA configuration parameter #########
		#
		source $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		# ######### Reload global functions #########
		#
		source $ELMA_LIB_DIR/elma-global-functions.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		library_version ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	fi

	return $RETVAL

}

# ######### Update openSUSE packages #########
#
update_opensuse_packages(){

	local RETVAL=0
	SUSE_UPDATE_OUTPUT=""
    SUSE_DIST_UPDATE_OUTPUT=""

	# ######### Reconfigure system distribution repositories #########
	#
	reconfigure_update_repositories ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	
	if [ ! -z $INTERNET_ACCESS ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Update openSUSE packages -----------------------------------------------"
		echo
		zypper -n --no-gpg-checks --gpg-auto-import-keys refresh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo

		SUSE_UPDATE_OUTPUT="$(zypper -n --no-gpg-checks --gpg-auto-import-keys update)" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		echo "$SUSE_UPDATE_OUTPUT"
		echo
        
		echo "--- `date +%T.%N | cut -c 1-12` - Distribution Upgrade ---------------------------------------------------"
		echo
		if [ "$ELMA_DEVEL_REPOSITORY_ENABLED" = "Yes" ]
		then
			SUSE_DIST_UPDATE_OUTPUT="$(zypper -n --no-gpg-checks --gpg-auto-import-keys dup --repo $ELMA_REPOSITORY_DEVEL)" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		else
			SUSE_DIST_UPDATE_OUTPUT="$(zypper -n --no-gpg-checks --gpg-auto-import-keys dup --repo $ELMA_REPOSITORY_MASTER)" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		echo "$SUSE_DIST_UPDATE_OUTPUT"
		echo

		# ######### Check differences between system and distribution config files #########
		#
		check_config_file_diffs ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		# ######### Reload ELMA configuration parameter #########
		#
		source $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		# ######### Reload global functions #########
		#
		source $ELMA_LIB_DIR/elma-global-functions.sh ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		library_version ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	fi

	return $RETVAL

}


# ######### Install and update required openSUSE packages #########
#
install_opensuse_packages(){

	local RETVAL=0
	SUSE_UPDATE_OUTPUT=""
	
	if [ ! -z $INTERNET_ACCESS ]
	then
		# ######### Reconfigure system distribution repositories #########
		#
		reconfigure_update_repositories ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		echo "--- `date +%T.%N | cut -c 1-12` - Remove rsyslog plugins with wrong dependencies -------------------------"
		echo
		zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive remove rsyslog-module-gssapi ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		echo "--- `date +%T.%N | cut -c 1-12` - Install required ELMA packages -----------------------------------------"
		echo
		zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive install --force elma elma-doc elma-tests elma-yast2 elma-rsyslog ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo

		if [ "$APACHE2_ENABLED" = "Yes" ]
		then
			echo "--- `date +%T.%N | cut -c 1-12` - Install required ELMA www packages -------------------------------------"
			echo
			zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive install --force elma-www ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		fi

		if [ "$WEBYAST_ENABLED" = "Yes" ]
		then
			echo "--- `date +%T.%N | cut -c 1-12` - Install required webyast packages --------------------------------------"
			echo
			zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive install --force elma-webyast ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		fi

		if [ "$KIBANA3_ENABLED" = "Yes" ]
		then
			echo "--- `date +%T.%N | cut -c 1-12` - Install required elasticsearch-kibana packages -------------------------"
			echo
			zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive install --force elma-elasticsearch-kibana ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		fi

		if [ "$ES_ENABLED" = "Yes" ]
		then
			echo "--- `date +%T.%N | cut -c 1-12` - Install required elasticsearch packages --------------------------------"
			echo
			zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive install --force elma-elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		fi

#		if [ "$LOGSTASH_ENABLED" = "Yes" ]
#		then
#			echo "--- `date +%T.%N | cut -c 1-12` - Install required Logstash packages -------------------------------------"
#			echo
#			zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive install --force elma-logstash ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#			zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive install --force logstash ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#			echo
#		fi

		if [ "$NAGIOS_ENABLED" = "Yes" ]
		then
			# issue 188 (wrong nagios-plugins-common dependencies)
			# nagios-plugins-linux_raid \
			#
			echo "--- `date +%T.%N | cut -c 1-12` - Remove Nagios plugins with wrong dependencies --------------------------"
			echo
			zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive remove nagios-plugins-linux_raid ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			echo "--- `date +%T.%N | cut -c 1-12` - Install and update required Nagios Core and extension packages ---------"
			echo
			zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive install --force elma-nagios ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

#			echo "--- `date +%T.%N | cut -c 1-12` - Install and update required Nagios Core remote clients -----------------"
#			echo
#			zypper -qn --no-gpg-checks --gpg-auto-import-keys --non-interactive install --force nsca nsca-client nrpe nrpe-doc nagios-plugins-nrpe ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		fi
	fi

	# ######### Patch and Update openSUSE packages #########
	#
	patch_opensuse_packages ; if [ $? -ne 0 ] ; then error_exit "$LINENO"; fi
	update_opensuse_packages ; if [ $? -ne 0 ] ; then error_exit "$LINENO"; fi

	return $RETVAL

}


# #########################################################################################
#
# ELMA installation and update functions
#
# #########################################################################################


# ######### Change initial appliance root password #########
#
change_root_password(){

	local RETVAL=0

	echo "--- `date +%T.%N | cut -c 1-12` - Change initial appliance root password ---------------------------------"
	echo
	read -p "Would you like to change the initial appliance root password [(Y)es or (N)o]? " INPUT
	echo
	if [ "$INPUT" = "Y" ] || [ "$INPUT" = "y" ] || [ "$INPUT" = "Yes" ] || [ "$INPUT" = "yes" ]
	then
		while :
		do
			passwd
			if [ $? -eq 0 ]
			then
				break
			else
				echo
				echo "Please check above error messages and try again:"
				echo
			fi
		done
	else
		echo "You can change your root password with:"
		echo "passwd"
	fi
	echo
	read -p "Press <ENTER> to continue... "
	echo

	return $RETVAL

}

# ######### Check openSUSE proxy configuration and internet access #########
#
check_internet_access(){

	local RETVAL=0
	INTERNET_ACCESS=""
	local STANDARD_OUTPUT="$1"

	if [ -z "$STANDARD_OUTPUT" ]; then echo "--- `date +%T.%N | cut -c 1-12` - Check openSUSE proxy configuration and internet access -----------------"; fi
	echo

	local URL="http://software.opensuse.org/"
	local URL_RESPONSE=$(wget -t 1 -T 15 -q -O - http://software.opensuse.org/)

	if [ -z "$URL_RESPONSE" ]
	then
		if [ -z "$STANDARD_OUTPUT" ] || [ "$STANDARD_OUTPUT" = "status" ]
		then
			echo "Could not reach openSUSE repositories on $URL !!!"
			echo
			echo "Please check your:"
			echo
			echo "- openSUSE proxy configuration"
			echo "- DNS resolving"
			echo "- internet connectivity"
			echo

			RETVAL=1
		else
			RETVAL=0
		fi
	else
		INTERNET_ACCESS="Successful"
		if [ "$STANDARD_OUTPUT" = "status" ]
		then
			echo "openSUSE repositories are reachable"
			echo
			grep "^PROXY_ENABLED=" $SYSCONFIG_PROXY
			grep "^HTTP_PROXY=" $SYSCONFIG_PROXY
			grep "^HTTPS_PROXY=" $SYSCONFIG_PROXY
			grep "^FTP_PROXY=" $SYSCONFIG_PROXY
			grep "^NO_PROXY=" $SYSCONFIG_PROXY
		fi

		RETVAL=0
	fi

	return $RETVAL

}

# ######### Check differences between system and distribution config files #########
#
check_config_file_diffs(){

	local RETVAL=0

	local STANDARD_OUTPUT="$1"

	local CONFIG_FILE_SYSTEM=$ELMA_CONF
	local CONFIG_FILE_SYSTEM_BACKUP=$ELMA_CONF.old
	local CONFIG_FILE_ELMA_DIST=$ELMA_CONF_DIR/elma.conf.org

	local CONFVAR
	local CONFVALUE

	if [ -z "$STANDARD_OUTPUT" ]; then echo "--- `date +%T.%N | cut -c 1-12` - Check differences between system and distribution configuration files --"; fi
	echo

	if [ -f $CONFIG_FILE_SYSTEM ]
	then
    		# Apply base directories for packages to ELMA config file
		#
		sed -i -e '/^INSTALLDIR=/s|=.*|="/opt"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^INITD_DIR=/s|=.*|="/etc/init.d"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^CRONTAB=/s|=.*|="/etc/crontab"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^LOGROTATE_DIR=/s|=.*|="/etc/logrotate.d"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^VAR_LOG=/s|=.*|="/var/log"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^VAR_LOCK=/s|=.*|="/var/lock"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^SBIN_DIR=/s|=.*|="/sbin"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^USR_SBIN_DIR=/s|=.*|="/usr/sbin"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^TMPDIR=/s|=.*|="/dev/shm"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^SYSCONFIG_DIR=/s|=.*|="/etc/sysconfig"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^SYSCONFIG_PROXY=/s|=.*|="$SYSCONFIG_DIR/proxy"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^SYSCONFIG_FIREWALL=/s|=.*|="$SYSCONFIG_DIR/SuSEfirewall2"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^SYSCONFIG_SYSLOG=/s|=.*|="$SYSCONFIG_DIR/syslog"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^SYSCONFIG_BACKUP_DIR=/s|=.*|="/srv/ftp/backups"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^SECURITY_LIMITS=/s|=.*|="/etc/security/limits.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^PERMISSIONS_LOCAL=/s|=.*|="/etc/permissions.local"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^BOOT_LOCAL=/s|=.*|="$INITD_DIR/boot.local"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^JAVA=/s|=.*|="/usr/bin/java"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^APACHE2_CONF_DIR=/s|=.*|="/etc/apache2"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^APACHE2_CONF_VHOSTS_DIR=/s|=.*|="$APACHE2_CONF_DIR/vhosts.d"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^APACHE2_CONF_CONFD_DIR=/s|=.*|="$APACHE2_CONF_DIR/conf.d"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^APACHE2_CONF_LISTEN=/s|=.*|="$APACHE2_CONF_DIR/listen.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^APACHE2_CONF_REDIRECT_HTTP=/s|=.*|="$APACHE2_CONF_CONFD_DIR/redirect-http.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^APACHE2_CONF_DEFAULT_AUTHENTICATION=/s|=.*|="$APACHE2_CONF_CONFD_DIR/default-authentication.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^APACHE2_CONF_DEFAULT_SSL=/s|=.*|="$APACHE2_CONF_CONFD_DIR/default-ssl.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^APACHE2_HTPASSWD_FILE=/s|=.*|="$APACHE2_CONF_CONFD_DIR/htpasswd"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^NAGIOS_DIR=/s|=.*|="/usr/share/nagios"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^NAGIOS_THEME_DIR=/s|=.*|="/usr/share/nagios-themes/exfoliation"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^NAGIOS_CONF_DIR=/s|=.*|="/etc/nagios"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^NAGIOS_CGI_DIR=/s|=.*|="/usr/lib/nagios/cgi"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^NAGIOS_PLUGINS_DIR=/s|=.*|="/usr/lib/nagios/plugins"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^NAGIOS_APACHE_CONF=/s|=.*|="$APACHE2_CONF_CONFD_DIR/nagios.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^NAGIOSGRAPH_CONF_DIR=/s|=.*|="/etc/nagiosgraph"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^RSYSLOG_TESTTOOLS_DIR=/s|=.*|="/usr/sbin"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^RSYSLOG_DIR=/s|=.*|="/opt/rsyslog-$RSYSLOG_VERSION"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^RSYSLOG_CONF=/s|=.*|="/etc/rsyslog.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^RSYSLOG_CONF_DIR=/s|=.*|="/etc/rsyslog.d"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^RSYSLOG_CONF_REMOTE=/s|=.*|="$RSYSLOG_CONF_DIR/remote.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^ELMA_DIR=/s|=.*|="/usr/share/elma"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ELMA_BIN_DIR=/s|=.*|="$ELMA_DIR/bin"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ELMA_LIB_DIR=/s|=.*|="$ELMA_DIR/lib"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
        sed -i -e '/^ELMA_CONF_DIR=/s|=.*|="/etc/elma"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ELMA_CONF=/s|=.*|="$ELMA_CONF/elma.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ELMA_TESTS_DIR=/s|=.*|="/usr/share/elma-tests"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ELMA_TESTS_BIN_DIR=/s|=.*|="$ELMA_TESTS_DIR/bin"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ELMA_TESTS_LIB_DIR=/s|=.*|="$ELMA_TESTS_DIR/lib"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
        sed -i -e '/^ELMA_DEVEL_DIR=/s|=.*|="/usr/share/elma-devel"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ELMA_DEVEL_BIN_DIR=/s|=.*|="$ELMA_DEVEL_DIR/bin"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ELMA_DEVEL_LIB_DIR=/s|=.*|="$ELMA_DEVEL_DIR/lib"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
        sed -i -e '/^ELMA_WWW_DIR=/s|=.*|="/srv/www/htdocs/elma"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ELMA_WWW_IMAGES_DIR=/s|=.*|="$ELMA_WWW_DIR/images"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ELMA_FTP_DIR=/s|=.*|="/srv/ftp"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^ES_DIR=/s|=.*|="/usr/share/java/elasticsearch"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ES_BIN=/s|=.*|="$ES_DIR/bin/service/elasticsearch"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ES_CONF_DIR=/s|=.*|="/etc/elasticsearch"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ES_CONF=/s|=.*|="$ES_CONF_DIR/elasticsearch.yml"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ES_CONF_LOGGING=/s|=.*|="$ES_CONF_DIR/logging.yml"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ES_APACHE_CONF=/s|=.*|="$APACHE2_CONF_CONFD_DIR/elasticsearch.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ES_LOG_DIR=/s|=.*|="$VAR_LOG/elasticsearch"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ES_LOCK_DIR=/s|=.*|="$VAR_LOCK/elasticsearch"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^ES_WORK_DIR=/s|=.*|="/tmp/elasticsearch"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^KIBANA3_DIR=/s|=.*|="/srv/www/htdocs/elasticsearch-kibana"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^KIBANA3_CONF=/s|=.*|="$KIBANA3_DIR/config.js"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^KIBANA3_APACHE_CONF=/s|=.*|="$APACHE2_CONF_CONFD_DIR/kibana3.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		sed -i -e '/^LOGSTASH_DIR=/s|=.*|="/opt/logstash-$LOGSTASH_VERSION"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^LOGSTASH_BIN=/s|=.*|="$LOGSTASH_DIR/logstash-$LOGSTASH_VERSION-monolithic.jar"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^LOGSTASH_CONF_DIR=/s|=.*|="/etc/logstash"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^LOGSTASH_CONF=/s|=.*|="$LOGSTASH_CONF_DIR/logstash-json-tcp-input.conf"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^LOGSTASH_LOG=/s|=.*|="$VAR_LOG/logstash.log"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^LOGSTASH_PID=/s|=.*|="/var/run/logstash.pid"|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
        
		cp $CONFIG_FILE_SYSTEM $CONFIG_FILE_SYSTEM_BACKUP ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $CONFIG_FILE_ELMA_DIST ]
	then
		cp $CONFIG_FILE_ELMA_DIST $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	grep -v '^#' $CONFIG_FILE_ELMA_DIST | grep . | while read LINE
	do
		CONFVAR="$(echo $LINE | awk -F'=' '{print $1}')" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		CONFVALUE="$(grep ^$CONFVAR= $CONFIG_FILE_SYSTEM_BACKUP | awk -F"\x27" '{print $2}')" ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		if [ ! $CONFVAR == "TODAY" ]  && [ ! $CONFVAR == "YESTERDAY" ]
		then
			if [ ! -z "$CONFVALUE" ]
			then
				sed -i -e '/^'"$CONFVAR"'=/s|=.*|=\x27'"$CONFVALUE"'\x27|' $CONFIG_FILE_SYSTEM ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi
		fi
	done

	if [ "$STANDARD_OUTPUT" = "status" ]
	then
		echo "Differences between your and distribution configuration file:"
		echo
		echo "$CONFIG_FILE_SYSTEM                                           | $CONFIG_FILE_ELMA_DIST"
		echo
		diff --suppress-common-lines -a -y -W 130 $CONFIG_FILE_SYSTEM $CONFIG_FILE_ELMA_DIST ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	return $RETVAL

}

# ######### Check for system config file patches #########
#
check_config_file_patches(){

	local RETVAL=0

	if [ -s $ELMA_DIR/patches/README.work ]
	then
		mv $ELMA_DIR/patches/README.work $ELMA_DIR/patches/README ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		echo "Notes:"
		echo "------"
		echo
		echo "- Please check the differences between your and distribution config file:"
		echo
		cat $ELMA_DIR/patches/README ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	return $RETVAL

}

# ######### Reboot recommendations after openSUSE updates #########
#
reboot_recommendations(){

	local RETVAL=0

	REBOOT_RECOMMENDATIONS=$(echo "$SUSE_UPDATE_OUTPUT" | grep "Nothing to do")

	if [ "$SUSE_UPDATE_OUTPUT" ] && [ -z "$REBOOT_RECOMMENDATIONS" ]
	then
		echo
		echo "openSUSE Updates"
		echo "----------------"
		echo
		echo -e "$SUSE_UPDATE_OUTPUT"
		echo
		echo "- Update of required openSUSE packages was successful:"
		echo "  - Read carefully the openSUSE update warnings"
		echo "  - Consider to reboot the system"
	fi

	return $RETVAL

}

# ######### Update recommendations and availability checks of new versions #########
#
update_recommendations(){

	local RETVAL=0
	local UPDATE_RECOMMENDATIONS=""

	if [ ! -z $INTERNET_ACCESS ]
	then
		echo "Update Recommendations"
		echo "----------------------"
		echo

		zypper --gpg-auto-import-keys lu
		echo

#		search_newest_version "$GOOGLECODE_WIKI/Changelog" "Enterprise_Log_Management_Appliance" "noecho";
#		if [ ! "$NEWESTVERSION" = "$ELMA_APPLIANCE_VERSION" ] && [ $(date -d "${ELMA_APPLIANCE_VERSION_DATE}" +%s) -lt $(date -d "${NEWESTVERSION_DATE}" +%s) ]
#		then
#			echo "Appliance build   : $ELMA_APPLIANCE_VERSION ($ELMA_APPLIANCE_VERSION_DATE) -> $NEWESTVERSION ($NEWESTVERSION_DATE)"
#			UPDATE_RECOMMENDATIONS="$UPDATE_RECOMMENDATIONS\n\n- Update your ELMA appliance:"
#			UPDATE_RECOMMENDATIONS="$UPDATE_RECOMMENDATIONS\n  There will be no further support for ELMA appliances lower than 0.0.148.\n  It's strongly recommended to upgrade all of your ELMA appliances!\n  Upgrade howtos can be found on:\n  http://code.google.com/p/enterprise-log-management-appliance/wiki/FAQ"
#		fi
#
#		search_newest_version "$GOOGLECODE_WIKI/Changelog" "Based on " "noecho";
#		if [ ! "$NEWESTVERSION" = "$OPENSUSE_VERSION" ] && [ $(date -d "${OPENSUSE_VERSION_DATE}" +%s) -lt $(date -d "${NEWESTVERSION_DATE}" +%s) ]
#		then
#			echo "OS                : $OPENSUSE_VERSION ($OPENSUSE_VERSION_DATE) -> $NEWESTVERSION ($NEWESTVERSION_DATE)"
#			UPDATE_RECOMMENDATIONS="$UPDATE_RECOMMENDATIONS\n\n- Update your openSuse distribution:"
#			UPDATE_RECOMMENDATIONS="$UPDATE_RECOMMENDATIONS\n  There will be no further support for openSUSE 12.2 appliances.\n  It's strongly recommended to upgrade all of your ELMA appliances!\n  Upgrade howtos can be found on:\n  http://code.google.com/p/enterprise-log-management-appliance/wiki/FAQ"
#		fi
#
#		if [[ ! -z $UPDATE_RECOMMENDATIONS ]]
#		then
#			echo -e $UPDATE_RECOMMENDATIONS
#		fi
	fi

	return $RETVAL

}

# ######### Check disk space usage #########
#
check_disk_usage(){

	local RETVAL=0

	echo "Disk space usage:"
	echo
	df -h
	echo
	echo "Inode usage:"
	echo
	df -ih
	echo
	echo "-------------------------------------------------------------------------------------------"
	echo
	echo "Rsyslog spool directory and usage: /var/spool/rsyslog"
	echo
	echo "Usage	Dir"
	cd /var/spool; du -sch rsyslog
	echo
	df -h /var/spool
	echo
	df -ih /var/spool
	echo
	echo "-------------------------------------------------------------------------------------------"
	if [ "$ES_ENABLED" = "Yes" ]
	then
		echo
		echo "ElasticSearch data partition and usage: $ES_DATA_DIR"
		echo
		echo "Usage	Dir"
		cd $ES_DATA_DIR; du -sch *
		echo
		df -h $ES_DATA_DIR
		echo
		df -ih $ES_DATA_DIR
		echo
	fi
	echo "==========================================================================================="
	echo

	return $RETVAL

}

# ######### Check status of WebYaST #########
#
check_webyast_status(){

	local RETVAL=0
	local PID=""
	WEBYAST_STATUS=""

	WEBYAST_VERSION=$(rpm -q webyast-base | sed 's/\ .*$//;s/.\///;s/webyast-base-//')
	WEBYAST_VERSION_DATE=$(rpm -qi webyast-base | grep "Build Date" | awk -F' : ' '{print $2}')
	WEBYAST_VERSION_DATE=$(date -d "$WEBYAST_VERSION_DATE" "+%Y-%m-%d")

	if [ $WEBYAST_ENABLED = "Yes" ]
	then
		if [ "$WEBYAST_VERSION" ]
		then
			PID=$(pgrep -of /etc/webyast/nginx.conf)
			if [ $PID ]
			then
				WEBYAST_STATUS="$WEBYAST_VERSION ($WEBYAST_VERSION_DATE) - running with PID $PID"
			else
				WEBYAST_STATUS="$WEBYAST_VERSION ($WEBYAST_VERSION_DATE) - NOT running!!!"
			fi
		else
			WEBYAST_STATUS="NOT installed"
		fi
	else
		WEBYAST_STATUS="NOT enabled"
	fi

	return $RETVAL

}

# ######### Check status of collectd #########
#
check_collectd_status(){

	local RETVAL=0
	local PID=""
	COLLECTD_STATUS=""

	COLLECTD_VERSION=$(rpm -q collectd | sed 's/\ .*$//;s/.\///;s/collectd-//')
	COLLECTD_VERSION_DATE=$(rpm -qi collectd | grep "Build Date" | awk -F' : ' '{print $2}')
	COLLECTD_VERSION_DATE=$(date -d "$COLLECTD_VERSION_DATE" "+%Y-%m-%d")

	if [ $COLLECTD_ENABLED = "Yes" ]
	then
		if [ "$COLLECTD_VERSION" ]
		then
			PID=$(pgrep -of /usr/sbin/collectd)
			if [ $PID ]
			then
				COLLECTD_STATUS="$COLLECTD_VERSION ($COLLECTD_VERSION_DATE) - running with PID $PID"
			else
				COLLECTD_STATUS="$COLLECTD_VERSION ($COLLECTD_VERSION_DATE) - NOT running!!!"
			fi
		else
			COLLECTD_STATUS="NOT installed"
		fi
	else
		COLLECTD_STATUS="NOT enabled"
	fi

	return $RETVAL

}

# ######### Check status of FTP #########
#
check_ftp_status(){

	local RETVAL=0
	local PID=""
	FTP_STATUS=""

	FTP_VERSION=$(rpm -q vsftpd | sed 's/\ .*$//;s/.\///;s/vsftpd-//')
	FTP_VERSION_DATE=$(rpm -qi vsftpd | grep "Build Date" | awk -F' : ' '{print $2}')
	FTP_VERSION_DATE=$(date -d "$FTP_VERSION_DATE" "+%Y-%m-%d")

	if [ $FTP_ENABLED = "Yes" ]
	then
		if [ "$FTP_VERSION" ]
		then
			PID=$(pgrep -of /etc/vsftpd.conf)
			if [ $PID ]
			then
				FTP_STATUS="$FTP_VERSION ($FTP_VERSION_DATE) - running with PID $PID"
			else
				FTP_STATUS="$FTP_VERSION ($FTP_VERSION_DATE) - NOT running!!!"
			fi
		else
			FTP_STATUS="NOT installed"
		fi
	else
		FTP_STATUS="NOT enabled"
	fi

	return $RETVAL

}

# ######### Check status of elma #########
#
check_elma_status(){

	local RETVAL=0
	local PID=""
	ELMA_STATUS=""

	ELMA_VERSION=$(rpm -q elma | sed 's/\ .*$//;s/.\///;s/elma-//')
	ELMA_VERSION_DATE=$(rpm -qi elma | grep "Build Date" | awk -F' : ' '{print $2}')
	ELMA_VERSION_DATE=$(date -d "$ELMA_VERSION_DATE" "+%Y-%m-%d")

	if [ "$ELMA_VERSION" ]
	then
		ELMA_STATUS="$ELMA_VERSION ($ELMA_VERSION_DATE)"
	else
		ELMA_STATUS="NOT installed"
	fi

	return $RETVAL

}

# ######### Check status of rsyslog #########
#
check_rsyslog_status(){

	local RETVAL=0
	local PID=""
	RSYSLOG_STATUS=""

	RSYSLOG_VERSION=$(rpm -q rsyslog | sed 's/\ .*$//;s/.\///;s/rsyslog-//')
	RSYSLOG_VERSION_DATE=$(rpm -qi rsyslog | grep "Build Date" | awk -F' : ' '{print $2}')
	RSYSLOG_VERSION_DATE=$(date -d "$RSYSLOG_VERSION_DATE" "+%Y-%m-%d")

	if [ "$RSYSLOG_VERSION" ]
	then
		PID=$(pgrep -of /usr/sbin/rsyslogd)
		if [ $PID ]
		then
			RSYSLOG_STATUS="$RSYSLOG_VERSION ($RSYSLOG_VERSION_DATE) - running with PID $PID"
		else
			RSYSLOG_STATUS="$RSYSLOG_VERSION ($RSYSLOG_VERSION_DATE) - NOT running!!!"
		fi
	else
		RSYSLOG_STATUS="NOT installed"
	fi

	return $RETVAL

}

# ######### Check status of rsyslog testtools #########
#
check_rsyslog_testtools_status(){

	local RETVAL=0
	local PID=""
	RSYSLOG_TESTTOOLS_STATUS=""

	RSYSLOG_TESTTOOLS_VERSION=$(rpm -q rsyslog-testtools | sed 's/\ .*$//;s/.\///;s/rsyslog-testtools-//')
	RSYSLOG_TESTTOOLS_VERSION_DATE=$(rpm -qi rsyslog-testtools | grep "Build Date" | awk -F' : ' '{print $2}')
	RSYSLOG_TESTTOOLS_VERSION_DATE=$(date -d "$RSYSLOG_TESTTOOLS_VERSION_DATE" "+%Y-%m-%d")

	if [ "$RSYSLOG_TESTTOOLS_VERSION" ]
	then
		RSYSLOG_TESTTOOLS_STATUS="$RSYSLOG_TESTTOOLS_VERSION ($RSYSLOG_TESTTOOLS_VERSION_DATE)"
	else
		RSYSLOG_TESTTOOLS_STATUS="NOT installed"
	fi

	return $RETVAL

}

# ######### Check status of ElasticSearch #########
#
check_elasticsearch_status(){

	local RETVAL=0
	local PID=""
	ES_STATUS=""

	ES_VERSION=$(rpm -q elasticsearch | sed 's/\ .*$//;s/.\///;s/elasticsearch-//')
	ES_VERSION_DATE=$(rpm -qi elasticsearch | grep "Build Date" | awk -F' : ' '{print $2}')
	ES_VERSION_DATE=$(date -d "$ES_VERSION_DATE" "+%Y-%m-%d")

	if [ $ES_ENABLED = "Yes" ]
	then
		if [ "$ES_VERSION" ]
		then
			PID=$(pgrep -of elasticsearch)
			if [ $PID ]
			then
				ES_STATUS="$ES_VERSION ($ES_VERSION_DATE) - running with PID $PID"
			else
				ES_STATUS="$ES_VERSION ($ES_VERSION_DATE) - NOT running!!!"
			fi
		else
			ES_STATUS="NOT installed"
		fi
	else
		ES_STATUS="NOT enabled"
	fi

	return $RETVAL

}

# ######### Check status of Kibana3 #########
#
check_kibana3_status(){

	local RETVAL=0
	local PID=""
	KIBANA3_STATUS=""

	KIBANA3_VERSION=$(rpm -q elasticsearch-kibana | sed 's/\ .*$//;s/.\///;s/elasticsearch-kibana-//')
	KIBANA3_VERSION_DATE=$(rpm -qi elasticsearch-kibana | grep "Build Date" | awk -F' : ' '{print $2}')
	KIBANA3_VERSION_DATE=$(date -d "$KIBANA3_VERSION_DATE" "+%Y-%m-%d")

	if [ $KIBANA3_ENABLED = "Yes" ]
	then
		if [ "$KIBANA3_VERSION" ]
		then
			KIBANA3_STATUS="$KIBANA3_VERSION ($KIBANA3_VERSION_DATE)"
		else
			KIBANA3_STATUS="NOT installed"
		fi
	else
		KIBANA3_STATUS="NOT enabled"
	fi

	return $RETVAL

}

# ######### Check status of Nagios Core #########
#
check_nagios_status(){

	local RETVAL=0
	local PID=""
	NAGIOS_STATUS=""

	NAGIOS_CORE_VERSION=$(rpm -q nagios | sed 's/\ .*$//;s/.\///;s/nagios-//')
	NAGIOS_CORE_VERSION_DATE=$(rpm -qi nagios | grep "Build Date" | awk -F' : ' '{print $2}')
	NAGIOS_CORE_VERSION_DATE=$(date -d "$NAGIOS_CORE_VERSION_DATE" "+%Y-%m-%d")

	if [ $NAGIOS_ENABLED = "Yes" ]
	then
		if [ "$NAGIOS_CORE_VERSION" ]
		then
			PID=$(pgrep -of /usr/sbin/nagios)
			if [ $PID ]
			then
				NAGIOS_STATUS="$NAGIOS_CORE_VERSION ($NAGIOS_CORE_VERSION_DATE) - running with PID $PID"
			else
				NAGIOS_STATUS="$NAGIOS_CORE_VERSION ($NAGIOS_CORE_VERSION_DATE) - NOT running!!!"
			fi
		else
			NAGIOS_STATUS="NOT installed"
		fi
	else
			NAGIOS_STATUS="NOT enabled"
	fi

	return $RETVAL

}

# ######### Check status of apache #########
#
check_apache_status(){

	local RETVAL=0
	local PID=""
	APACHE_STATUS=""

	APACHE2_VERSION=$(rpm -q apache2 | sed 's/\ .*$//;s/.\///;s/apache2-//')
	APACHE2_VERSION_DATE=$(rpm -qi apache2 | grep "Build Date" | awk -F' : ' '{print $2}')
	APACHE2_VERSION_DATE=$(date -d "$APACHE2_VERSION_DATE" "+%Y-%m-%d")

	if [ $APACHE2_ENABLED = "Yes" ]
	then
		if [ "$APACHE2_VERSION" ]
		then
			PID=$(pgrep -of /usr/sbin/httpd2-prefork)
			if [ $PID ]
			then
				APACHE_STATUS="$APACHE2_VERSION ($APACHE2_VERSION_DATE) - running with PID $PID"
			else
				APACHE_STATUS="$APACHE2_VERSION ($APACHE2_VERSION_DATE) - NOT running!!!"
			fi
		else
			APACHE_STATUS="NOT installed"
		fi
	else
			APACHE_STATUS="NOT enabled"
	fi

	return $RETVAL

}

# ######### Check status of openSuse #########
#
check_opensuse_status(){

	local RETVAL=0
	OPENSUSE_STATUS=""

	OPENSUSE_VERSION=$(grep "PRETTY_NAME=" /etc/os-release | awk -F'=' '{print $2}' | awk -F'"' '{print $2}')
	OPENSUSE_VERSION_DATE=$(ls -l --time-style=+%Y-%m-%d /etc/os-release | awk -F' ' '{print $6}')

	if [ "$OPENSUSE_VERSION" ]
	then
		OPENSUSE_STATUS="Based on $OPENSUSE_VERSION ($OPENSUSE_VERSION_DATE)"
	else
		OPENSUSE_STATUS="NOT installed"
	fi

	return $RETVAL

}

# ######### Check status of ELMA appliance #########
#
check_appliance_status(){

	local RETVAL=0
	ELMA_APPLIANCE_STATUS=""

	ELMA_APPLIANCE_VERSION=$(cat /etc/ImageVersion)
	ELMA_APPLIANCE_VERSION_DATE=$(ls -l --time-style=+%Y-%m-%d /etc/ImageVersion | awk -F' ' '{print $6}')

	if [ "$ELMA_APPLIANCE_VERSION" ]
	then
		ELMA_APPLIANCE_STATUS="$ELMA_APPLIANCE_VERSION ($ELMA_APPLIANCE_VERSION_DATE)"
	else
		ELMA_APPLIANCE_STATUS="NOT installed"
	fi

	return $RETVAL

}

# ######### Check status of installed packages #########
#
check_packages_status(){

	local RETVAL=0

	local PID=""

	echo "Appliance build"
	echo "---------------"

	check_appliance_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo "$ELMA_APPLIANCE_STATUS"

	check_opensuse_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo "$OPENSUSE_STATUS"
	echo

	echo "Release"
	echo "-------"
	check_elma_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo "ELMA              : $ELMA_STATUS"
	echo

	echo "Log management services"
	echo "-----------------------"
	check_rsyslog_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo "rsyslog           : $RSYSLOG_STATUS"
	check_rsyslog_testtools_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi 
	echo "rsyslog testtools : $RSYSLOG_TESTTOOLS_STATUS"
	echo

	echo "Log indexing and search services"
	echo "--------------------------------"
	check_elasticsearch_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo "ElasticSearch     : $ES_STATUS"
	check_kibana3_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo "Kibana3           : $KIBANA3_STATUS"
	echo

	echo "Monitoring services"
	echo "-------------------"
	check_nagios_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo "Nagios Core       : $NAGIOS_STATUS"
	check_collectd_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo "collectd          : $COLLECTD_STATUS"
	echo

	echo "Web services"
	echo "------------"
	check_apache_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
       echo "Apache2           : $APACHE_STATUS"
	check_webyast_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
       echo "WebYaST           : $WEBYAST_STATUS"
	echo

	echo "FTP services"
	echo "------------"
	check_ftp_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
       echo "vsftp             : $FTP_STATUS"
	echo

	echo "==========================================================================================="
	echo

	return $RETVAL

}

# ######### Search newest package version on Google Code repository #########
#
#search_newest_version(){
#
#	local RETVAL=0
#	local URL=$1
#	local SEARCH_VALUE=$2
#	local SHOWECHO=$3
#
#	NEWESTVERSION=""
#	NEWESTVERSION_DATE=""
#
#	if [ -z "$SHOWECHO" ]
#	then
#		echo "--- `date +%T.%N | cut -c 1-12` - Search newest version --------------------------------------------------"
#		echo
#	fi
#
#	if [ "$SEARCH_VALUE" = "Enterprise_Log_Management_Appliance" ]
#	then
#		NEWESTVERSION=$(curl -s --connect-timeout 5 --insecure $PROXY_SETTINGS $URL  | grep "$SEARCH_VALUE" | awk -F ' ' '{print $7}')
#		NEWESTVERSION_DATE=$(curl -s --connect-timeout 5 --insecure $PROXY_SETTINGS $URL  | grep "$SEARCH_VALUE" | awk -F ' ' '{print $8}' | awk -F '(' '{print $2}'| awk -F ')' '{print $1}')
#	elif [ "$SEARCH_VALUE" = "Based on " ]
#	then
#		NEWESTVERSION=$(curl -s --connect-timeout 5 --insecure $PROXY_SETTINGS $URL  | grep "$SEARCH_VALUE" | awk -F ' ' '{print $3,$4,$5,$6;}')
#		NEWESTVERSION_DATE=$(curl -s --connect-timeout 5 --insecure $PROXY_SETTINGS $URL  | grep "$SEARCH_VALUE" | awk -F ' ' '{print $7}' | awk -F '(' '{print $2}'| awk -F ')' '{print $1}')
#	else
#		NEWESTVERSION=$(curl -s --connect-timeout 5 --insecure $PROXY_SETTINGS $URL  | grep "$SEARCH_VALUE" | awk -F ' - ' '{print $2}' | awk -F ' ' '{print $1}')
#		NEWESTVERSION_DATE=$(curl -s --connect-timeout 5 --insecure $PROXY_SETTINGS $URL  | grep "$SEARCH_VALUE" | awk -F ' - ' '{print $2}' | awk -F ' ' '{print $2}' | awk -F '(' '{print $2}'| awk -F ')' '{print $1}')
#	fi
#
#	if [ -z "$NEWESTVERSION" ]
#	then
#		echo "Could not reach ELMA Changelog on $GOOGLECODE_WIKI/Changelog !!!"
#		echo "Please check your openSUSE proxy configuration, DNS resolving or internet connectivity."
#		echo
#		RETVAL=1
#	else
#		if [ -z "$SHOWECHO" ]
#		then
#			echo "$SEARCH_VALUE$NEWESTVERSION"
#			echo
#		fi
#		RETVAL=0
#	fi
#
#	return $RETVAL
#
#}

# ######### Install and update Logstash #########
#
#install_logstash(){
#
#	local RETVAL=0
#
#	# ######### Newest stable Logstash version #########
#	#
#	NEWEST_STABLE_LOGSTASH_VERSION="logstash-1.1.9"
#
#	# ######### Logstash download and install parameter #########
#	#
#	PACKAGEFILE="$NEWEST_STABLE_LOGSTASH_VERSION-monolithic.jar"
#	PACKAGEURL="https://logstash.objects.dreamhost.com/release/$PACKAGEFILE"
#	PACKAGENAME="logstash"
#	LINKNAME="logstash"
#
#	# ######### Install and update logstash #########
#	#
#	echo "--- `date +%T.%N | cut -c 1-12` - Install and update logstash --------------------------------------------"
#	echo
#
#	if [ ! -d $INSTALLDIR/$NEWEST_STABLE_LOGSTASH_VERSION ]
#	then
#		mkdir $INSTALLDIR/$NEWEST_STABLE_LOGSTASH_VERSION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#		download_new_package $PACKAGEURL $INSTALLDIR/$NEWEST_STABLE_LOGSTASH_VERSION $PACKAGEFILE ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	fi
#
#	link_new_package $PACKAGENAME $INSTALLDIR $LINKNAME ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#
#	cd $DIR; find . -maxdepth 1 -name "$NAME*" -type d -exec stat -t {} \; | sort -r -n -k 13,13 | head --lines=1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#
#	LOGSTASH_VERSION=$(echo $NEWPACKAGE | sed 's/\ .*$//;s/.\///;s/logstash-//')
#	LOGSTASH_VERSION_DATE=$(cd $DIR; tar -tzvf $NEWTARFILE | head --lines=1 | awk -F' ' '{print $4}')
#	LOGSTASH_DIR="$INSTALLDIR/logstash-$LOGSTASH_VERSION"
#	LOGSTASH_BIN="$LOGSTASH_DIR/logstash-$LOGSTASH_VERSION-monolithic.jar"
#
#	# ######### Update logstash version variable #########
#	#
#	echo
#	echo "--- `date +%T.%N | cut -c 1-12` - Update logstash version variable ---------------------------------------"
#	echo
#	sed -i -e '/LOGSTASH_VERSION=/s/=.*/="'"$LOGSTASH_VERSION"'"/' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	sed -i -e '/LOGSTASH_VERSION_DATE=/s/=.*/="'"$LOGSTASH_VERSION_DATE"'"/' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	if [ $LOGGING -eq 1 ]; then grep "LOGSTASH_VERSION=" $ELMA_CONF >&4; fi
#	if [ $LOGGING -eq 1 ]; then grep "LOGSTASH_VERSION_DATE=" $ELMA_CONF >&4; fi
#	if [ $LOGGING -eq 1 ]; then echo >&4; fi
#
#	# ######### Configuring logstash #########
#	#
#	echo "--- `date +%T.%N | cut -c 1-12` - Configuring logstash ---------------------------------------------------"
#	echo
#
#	if [ ! -d $LOGSTASH_CONF_DIR ]
#	then
#		mkdir $LOGSTASH_CONF_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	fi
#	if [ ! -f $LOGSTASH_CONF_DIR/logstash-json-tcp-input.conf ]
#	then
#		cp $ELMA_DIR/config/logstash/logstash-json-tcp-input.conf $LOGSTASH_CONF_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	fi
#	if [ ! -f $LOGSTASH_CONF_DIR/logstash-json-udp-input.conf ]
#	then
#		cp $ELMA_DIR/config/logstash/logstash-json-udp-input.conf $LOGSTASH_CONF_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	fi
#
#	return $RETVAL
#
#}


# #########################################################################################
#
# openSUSE configure and reconfigure functions
#
# #########################################################################################

# ######### System reboot #########
#
system_reboot(){

	local RETVAL=0

	echo "--- `date +%T.%N | cut -c 1-12` - System reboot ----------------------------------------------------------"
	echo
	reboot -f &

	return $RETVAL

}

# ######### System diagnostics - File system checks and cleanup #########
#
system_diagnostics(){

	local RETVAL=0

	# ######### File system checks and cleanup #########
	#

	echo "--- `date +%T.%N | cut -c 1-12` - Check disk space usage -------------------------------------------------"
	echo
	df -h ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	echo "--- `date +%T.%N | cut -c 1-12` - Check inode usage ------------------------------------------------------"
	echo
	df -ih ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	#echo "--- `date +%T.%N | cut -c 1-12` - Cleanup tmp directory --------------------------------------------------"
	#echo
	##rm -Rf /tmp/* ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	#echo

	echo "--- `date +%T.%N | cut -c 1-12` - Check log partition ----------------------------------------------------"
	echo
	echo "Log partition and usage: /var/log"
	echo
	echo "Usage	Dir"
	cd /var/log; du -sch * ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo
	df -h /var/log ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	echo "--- `date +%T.%N | cut -c 1-12` - Check rsyslog spool directory ------------------------------------------"
	echo
	echo "Rsyslog spool directory and usage: /var/spool/rsyslog"
	echo
	echo "Usage	Dir"
	cd /var/spool; du -sch rsyslog ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo
	df -h /var/spool ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo
	df -ih /var/spool ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	echo "--- `date +%T.%N | cut -c 1-12` - Cleanup rsyslog spool directory ----------------------------------------"
	echo
	###
	### rm -Rf /var/spool/rsyslog ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	# find /var/spool -name 'rsyslog' |xargms rm -fR
	# mkdir /var/spool/rsyslog ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	ls /var/spool/rsyslog/* | xargs rm -fR
	echo

	if [ "$ES_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Check ElasticSearch data partition -------------------------------------"
		echo
		echo "ElasticSearch data partition and usage: $ES_DATA_DIR"
		echo
		echo "Usage	Dir"
		cd $ES_DATA_DIR; du -sch * ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
		df -h $ES_DATA_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
		df -ih $ES_DATA_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	fi

	# ######### Count files in all subdirectories under / directory #########
	# 
	echo "--- `date +%T.%N | cut -c 1-12` - Count files in all subdirectories under / directory --------------------"
	echo
	find / -xdev -type f | cut -d "/" -f 2 | sort | uniq -c | sort -n
	echo

	# ######### Time synchronization #########
	#
	reconfigure_ntp_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure rsyslog service #########
	#
	reconfigure_rsyslog_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Apache2 service #########
	#
	reconfigure_apache_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Snapshot of the current processes #########
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Snapshot of the current processes --------------------------------------"
	echo
	ps -ef ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo

	# ######### Check openSUSE proxy configuration and internet access #########
	#
	check_internet_access ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}

# ######### Reset ELMA configuration parameter to factory defaults #########
#
config_reset(){

	local RETVAL=0

	if [ -f $ELMA_CONF_DIR/elma.conf.org ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Reset ELMA configuration parameter to factory defaults -----------------"
		echo
		echo "Resetting ELMA configuration parameter..."
		echo
		cp $ELMA_CONF_DIR/elma.conf.org $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		echo "ELMA configuration file $ELMA_CONF_DIR/elma.conf.org NOT found!"
		RETVAL=1
	fi

	reconfigure_elma_services ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}

# ######### Reconfigure system distribution repositories #########
#
reconfigure_update_repositories(){

	local RETVAL=0

	echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure system package locks ---------------------------------------"
	echo

	if [ "$ELMA_PACKAGE_LOCK_ENABLED" = "Yes" ]
	then
		zypper --gpg-auto-import-keys addlock 'elma*' ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		zypper --gpg-auto-import-keys removelock 'elma*' ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	check_opensuse_status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ ! -z "$(echo $OPENSUSE_VERSION | grep "openSUSE 12.3")" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Check required openSUSE 12.3 update-repositories -----------------------"
		echo
		zypper --gpg-auto-import-keys removerepo home_heinemannj66
		zypper --gpg-auto-import-keys removerepo home_heinemannj66_elma-devel
		zypper --gpg-auto-import-keys removerepo heinemannj66_12.3
		zypper --gpg-auto-import-keys removerepo security:logging:elma:devel_12.3
		zypper --gpg-auto-import-keys addrepo --check --refresh --keep-packages http://download.opensuse.org/repositories/security:/logging:/elma/openSUSE_12.3/security:logging:elma.repo
		zypper --gpg-auto-import-keys addrepo --check --refresh --keep-packages http://download.opensuse.org/repositories/security:/logging:/elma:/devel/openSUSE_12.3/security:logging:elma:devel.repo
		zypper --gpg-auto-import-keys addrepo --check --refresh --keep-packages http://download.opensuse.org/repositories/devel:/languages:/haskell/openSUSE_12.3/devel:languages:haskell.repo
		echo
	fi

	if [ ! -z "$(echo $OPENSUSE_VERSION | grep "openSUSE 13.1")" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Check required openSUSE 13.1 update-repositories -----------------------"
		echo
		zypper --gpg-auto-import-keys removerepo home_heinemannj66
		zypper --gpg-auto-import-keys removerepo home_heinemannj66_elma-devel
        	zypper --gpg-auto-import-keys removerepo heinemannj66_13.1
		zypper --gpg-auto-import-keys removerepo security:logging:elma:devel_13.1
		zypper --gpg-auto-import-keys addrepo --check --refresh --keep-packages http://download.opensuse.org/repositories/security:/logging:/elma/openSUSE_13.1/security:logging:elma.repo
		zypper --gpg-auto-import-keys addrepo --check --refresh --keep-packages http://download.opensuse.org/repositories/security:/logging:/elma:/devel/openSUSE_13.1/security:logging:elma:devel.repo
		zypper --gpg-auto-import-keys addrepo --check --refresh --keep-packages http://download.opensuse.org/repositories/devel:/languages:/haskell/openSUSE_13.1/devel:languages:haskell.repo
		echo
	fi

	echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure system distribution repositories ---------------------------"
	echo

	if [ "$ELMA_DEVEL_REPOSITORY_ENABLED" = "Yes" ]
	then
		zypper --gpg-auto-import-keys modifyrepo --disable 'security_logging_elma' ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		zypper --gpg-auto-import-keys modifyrepo --enable 'security_logging_elma_devel' ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	else
		zypper --gpg-auto-import-keys modifyrepo --enable 'security_logging_elma' ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		zypper --gpg-auto-import-keys modifyrepo --disable 'security_logging_elma_devel' ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
	fi

	return $RETVAL

}

# ######### Reconfigure openSUSE proxy configuration #########
#
reconfigure_proxy_configuration(){

	local RETVAL=0

	echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure openSUSE proxy configuration -------------------------------"
	echo

	if [ "$PROXY_ENABLED" = "Yes" ]
	then
		sed -i -e '/^#PROXY_SETTINGS=".*"/s/#//' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^PROXY_SETTINGS=""/s/^/#/' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '/^PROXY_SETTINGS=".*"/s/^/#/' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^#PROXY_SETTINGS=""/s/#//' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ "$PROXY_AUTH_ENABLED" = "Yes" ]
	then
		sed -i -e '/^#PROXY_AUTH=".*"/s/#//' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^PROXY_AUTH=""/s/^/#/' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '/^PROXY_AUTH=".*"/s/^/#/' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^#PROXY_AUTH=""/s/#//' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	# ######### Reload ELMA configuration file #########
	#
	source $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ "$PROXY_SETTINGS" == "" ]; then
		sed -i -e '/^PROXY_ENABLED=/s/"[^"]*"/"No"/' $SYSCONFIG_PROXY ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^HTTP_PROXY=/s/"[^"]*"/""/' $SYSCONFIG_PROXY ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^HTTPS_PROXY=/s/"[^"]*"/""/' $SYSCONFIG_PROXY ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^FTP_PROXY=/s/"[^"]*"/""/' $SYSCONFIG_PROXY ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^NO_PROXY=/s/"[^"]*"/"localhost, 127.0.0.1"/' $SYSCONFIG_PROXY ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '/^PROXY_ENABLED=/s/"[^"]*"/"Yes"/' $SYSCONFIG_PROXY ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^HTTP_PROXY=/s/"[^"]*"/"'$PROXY_PROTOCOL':\/\/'$PROXY_HOST':'$PROXY_PORT'"/' $SYSCONFIG_PROXY ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^HTTPS_PROXY=/s/"[^"]*"/"'$PROXY_PROTOCOL':\/\/'$PROXY_HOST':'$PROXY_PORT'"/' $SYSCONFIG_PROXY ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^FTP_PROXY=/s/"[^"]*"/"'$PROXY_PROTOCOL':\/\/'$PROXY_HOST':'$PROXY_PORT'"/' $SYSCONFIG_PROXY ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^NO_PROXY=/s/"[^"]*"/"localhost, 127.0.0.1"/' $SYSCONFIG_PROXY ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	check_internet_access ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}

# ######### Reconfigure systemd service #########
#
reconfigure_systemd_service(){

	local RETVAL=0

	echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure systemd service --------------------------------------------"
	echo

	if [ -f journald.conf ]
	then
		sed -i -e '/#.*ForwardToSyslog=/s/#//' /etc/systemd/journald.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/ForwardToSyslog=/s/=.*/=yes/' /etc/systemd/journald.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ $LOGGING -eq 1 ]; then grep "ForwardToSyslog" /etc/systemd/journald.conf >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	echo "--- `date +%T.%N | cut -c 1-12` - Restarting systemd service ---------------------------------------------"
	echo
	systemctl restart systemd-journald.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	systemctl status systemd-journald.service
	echo

	return $RETVAL

}

# ######### Enable and reconfigure Network service #########
#
reconfigure_network_service(){

	local RETVAL=0

	echo "--- `date +%T.%N | cut -c 1-12` - Enable Network service -------------------------------------------------"
	echo
	systemctl is-enabled network.service >/dev/null 2>&1 ; RETVAL=$?
	echo >/dev/null 2>&1
	if [ $RETVAL -ne 0 ]
	then
		systemctl enable network.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
		RETVAL=0
	fi

	echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Network service --------------------------------------------"
	echo

	echo "--- `date +%T.%N | cut -c 1-12` - Restarting Network service ---------------------------------------------"
	echo
	systemctl is-enabled network.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo >/dev/null 2>&1
	systemctl restart network.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	systemctl status network.service
	echo

	return $RETVAL

}

# ######### Enable and reconfigure SuSEfirewall service #########
#
reconfigure_susefirewall_service(){

	local RETVAL=0

	if [ "$FW_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Enable SuSEfirewall service --------------------------------------------"
		echo
		systemctl is-enabled SuSEfirewall2.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			systemctl enable SuSEfirewall2.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure SuSEfirewall service ---------------------------------------"
		echo

		if [ -f $SYSCONFIG_FIREWALL ]
		then
			# ######### Open TCP ports #########
			#
			SYSCONFIG_FW_SERVICES_EXT_TCP=`grep "FW_SERVICES_EXT_TCP=" $SYSCONFIG_FIREWALL | grep -v "#" | awk -F\" '{print $2}'`
			FW_SERVICES_EXT_TCP=$(echo "$SYSCONFIG_FW_SERVICES_EXT_TCP $FW_SERVICES_EXT_TCP" | tr " " "\n" | sort -u | tr "\n" " " | sed 's/[ \t]*$//' | sed 's/^[ \t]*//')

			sed -i -e '/^.*FW_SERVICES_EXT_TCP=/s/".*"/"'"$FW_SERVICES_EXT_TCP"'"/' $SYSCONFIG_FIREWALL ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			if [ $LOGGING -eq 1 ]; then grep "FW_SERVICES_EXT_TCP=" $SYSCONFIG_FIREWALL | grep -v "#" >&4; fi

			# ######### Open UDP ports #########
			#
			SYSCONFIG_FW_SERVICES_EXT_UDP=`grep "FW_SERVICES_EXT_UDP=" $SYSCONFIG_FIREWALL | grep -v "#" | awk -F\" '{print $2}'`
			FW_SERVICES_EXT_UDP=$(echo "$SYSCONFIG_FW_SERVICES_EXT_UDP $FW_SERVICES_EXT_UDP" | tr " " "\n" | sort -u | tr "\n" " " | sed 's/[ \t]*$//' | sed 's/^[ \t]*//')

			sed -i -e '/^.*FW_SERVICES_EXT_UDP=/s/".*"/"'"$FW_SERVICES_EXT_UDP"'"/' $SYSCONFIG_FIREWALL ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			if [ $LOGGING -eq 1 ]; then grep "FW_SERVICES_EXT_UDP=" $SYSCONFIG_FIREWALL | grep -v "#" >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting SuSEfirewall service ----------------------------------------"
		echo
		systemctl is-enabled SuSEfirewall2.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo >/dev/null 2>&1
		systemctl restart SuSEfirewall2.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status SuSEfirewall2_init.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
		systemctl status SuSEfirewall2.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable SuSEfirewall service -------------------------------------------"
		echo
		systemctl is-enabled SuSEfirewall2.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable SuSEfirewall2.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop SuSEfirewall service ----------------------------------------------"
		echo
		systemctl is-active SuSEfirewall2.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1 
		if [ $RETVAL -eq 0 ]
		then
			systemctl stop SuSEfirewall2.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			systemctl status SuSEfirewall2.service
			echo
		else
			RETVAL=0
		fi
	fi

	return $RETVAL

}

# ######### Enable and reconfigure postfix service #########
#
reconfigure_postfix_service(){

	local RETVAL=0

	echo "--- `date +%T.%N | cut -c 1-12` - Enable postfix service -------------------------------------------------"
	echo
	systemctl is-enabled postfix.service >/dev/null 2>&1 ; RETVAL=$?
	echo >/dev/null 2>&1
	if [ $RETVAL -ne 0 ]
	then
		systemctl enable postfix.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo
		RETVAL=0
	fi

	echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure postfix service --------------------------------------------"
	echo

	if [ -f /etc/aliases ]
	then
		sed -i -e '/^#root:/s/#root:.*/root:		'$MAIL_REDIRECT', \\root/' /etc/aliases ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^root:/s/root:.*/root:		'$MAIL_REDIRECT', \\root/' /etc/aliases ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ $LOGGING -eq 1 ]; then grep "root:" /etc/aliases >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi
	/usr/bin/newaliases

	if [ ! -z "$MAIL_RELAYHOST" ]
	then
		if [ -f /etc/sysconfig/postfix ]
		then
			sed -i -e '/^.*POSTFIX_RELAYHOST=/s/=".*"/="'$MAIL_RELAYHOST'"/' /etc/sysconfig/postfix ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		if [ $LOGGING -eq 1 ]; then grep "POSTFIX_RELAYHOST=" /etc/sysconfig/postfix >&4; fi
		if [ $LOGGING -eq 1 ]; then echo >&4; fi
		/usr/sbin/config.postfix
		echo
	fi

	echo "--- `date +%T.%N | cut -c 1-12` - Restarting postfix service ---------------------------------------------"
	echo
	systemctl is-enabled postfix.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo >/dev/null 2>&1
	systemctl restart postfix.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	systemctl status postfix.service
	echo

	return $RETVAL

}

# ######### Enable and reconfigure SNMP service #########
#
reconfigure_snmpd_service(){

	local RETVAL=0
	local SNMP_AGENT_ADDRESS=""

	if [ "$SNMP_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Enable snmpd service ---------------------------------------------------"
		echo
		systemctl is-enabled snmpd.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			systemctl enable snmpd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure SNMP service -----------------------------------------------"
		echo
		if [ ! -z "$SNMP_READ_ONLY_COMMUNITY" ]
		then
			sed -i -e '/^rocommunity /s/ .*/ '$SNMP_READ_ONLY_COMMUNITY' '$SNMP_BIND'/' /etc/snmp/snmpd.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			if [ $LOGGING -eq 1 ]; then grep "rocommunity" /etc/snmp/snmpd.conf >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi
		fi

		if [ ! -z "$SNMP_BIND" ]
		then
			SNMP_AGENT_ADDRESS="$SNMP_BIND:"
		fi

		if [ -z "$(grep "agentAddress" /etc/snmp/snmpd.conf)" ]
		then
			echo -e "\nagentAddress udp:$SNMP_AGENT_ADDRESS161" >> /etc/snmp/snmpd.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		sed -i -e '/^agentAddress /s/ .*/ udp:'$SNMP_AGENT_ADDRESS'161/' /etc/snmp/snmpd.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		if [ $LOGGING -eq 1 ]; then grep "agentAddress" /etc/snmp/snmpd.conf >&4; fi
		if [ $LOGGING -eq 1 ]; then echo >&4; fi

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting SNMP service ------------------------------------------------"
		echo
		systemctl is-enabled snmpd.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo >/dev/null 2>&1
		systemctl restart snmpd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status snmpd.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable snmpd service --------------------------------------------------"
		echo
		systemctl is-enabled snmpd.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable snmpd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop snmpd service -----------------------------------------------------"
		echo
		systemctl is-active snmpd.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1 
		if [ $RETVAL -eq 0 ]
		then
			systemctl stop snmpd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			systemctl status snmpd.service
			echo
		else
			RETVAL=0
		fi
	fi

	# ######### Reconfigure Nagios Core #########
	#
	reconfigure_nagios_core_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}

# ######### Enable and reconfigure SNMP trap receiver service #########
#
reconfigure_snmptrapd_service(){

	local RETVAL=0

	if [ "$SNMP_TRAP_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Enable snmptrapd service -----------------------------------------------"
		echo
		systemctl is-enabled snmptrapd.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			systemctl enable snmptrapd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure SNMP trap receiver service ---------------------------------"
		echo

		if [ ! -z "$SNMP_READ_ONLY_COMMUNITY" ]
		then
			sed -i -e '/^authCommunity log,execute,net /s/authCommunity log,execute,net .*/authCommunity log,execute,net '$SNMP_READ_ONLY_COMMUNITY'/' /etc/snmp/snmptrapd.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			if [ $LOGGING -eq 1 ]; then grep "authCommunity" /etc/snmp/snmptrapd.conf >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi
		fi
		
		sed -i -e '/^.*startproc /s/startproc .*/startproc $SNMPTRAPD $SNMPTRAPD_CONF -Ot -On -OQ -p $SNMPTRAPD_PID/' /etc/init.d/snmptrapd ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl --system daemon-reload ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		if [ $LOGGING -eq 1 ]; then grep "startproc" /etc/init.d/snmptrapd >&4; fi
		if [ $LOGGING -eq 1 ]; then echo >&4; fi

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting SNMP trap receiver service ----------------------------------"
		echo
		systemctl is-enabled snmptrapd.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo >/dev/null 2>&1
		systemctl restart snmptrapd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status snmptrapd.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable snmptrapd service ----------------------------------------------"
		echo
		systemctl is-enabled snmptrapd.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable snmptrapd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop snmptrapd service -------------------------------------------------"
		echo
		systemctl is-active snmptrapd.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1 
		if [ $RETVAL -eq 0 ]
		then
			systemctl stop snmptrapd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			systemctl status snmptrapd.service
			echo
		else
			RETVAL=0
		fi
	fi

	return $RETVAL

}

# ######### Enable and reconfigure FTP service #########
#
reconfigure_ftp_service(){

	local RETVAL=0

	if [ "$FTP_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Enable FTP service -----------------------------------------------------"
		echo
		systemctl is-enabled vsftpd.service >/dev/null 2>&1 ; RETVAL=$?
		echo  >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			systemctl enable vsftpd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure FTP service ------------------------------------------------"
		echo

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting FTP service -------------------------------------------------"
		echo
		systemctl is-enabled vsftpd.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo >/dev/null 2>&1
		systemctl restart vsftpd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status vsftpd.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable FTP service ----------------------------------------------------"
		echo
		systemctl is-enabled vsftpd.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable vsftpd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop FTP service -------------------------------------------------------"
		echo
		systemctl is-active vsftpd.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1 
		if [ $RETVAL -eq 0 ]
		then
			systemctl stop vsftpd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			systemctl status vsftpd.service
			echo
		else
			RETVAL=0
		fi
	fi

	return $RETVAL

}

# ######### Enable and reconfigure collectd service #########
#
reconfigure_collectd_service(){

	local RETVAL=0

	if [ "$COLLECTD_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Enable collectd service ------------------------------------------------"
		echo
		systemctl is-enabled collectd.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			systemctl enable collectd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure collectd service -------------------------------------------"
		echo

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting collectd service --------------------------------------------"

		echo
		systemctl is-enabled collectd.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo >/dev/null 2>&1
		systemctl restart collectd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status collectd.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable collectd service -----------------------------------------------"

		echo
		systemctl is-enabled collectd.service >/dev/null 2>&1 ; RETVAL=$?
		ech0 >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable collectd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop collectd service --------------------------------------------------"
		echo
		systemctl is-active collectd.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl stop collectd.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			systemctl status collectd.service
			echo
		else
			RETVAL=0
		fi
	fi

	return $RETVAL

}

# ######### Enable and reconfigure NTP service #########
#
reconfigure_ntp_service(){

	local RETVAL=0
	local NTP_FORWARDER_ARRAY=( $NTP_FORWARDER )
	local NTP_FORWARDER_LIST

	typeset -i i=0 max=${#NTP_FORWARDER_ARRAY[*]}
	while (( i < max ))
	do
		NTP_FORWARDER_LIST=$NTP_FORWARDER_LIST"server ${NTP_FORWARDER_ARRAY[$i]}\n"
		i=i+1
	done

	if [ "$NTP_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Enable ntpd service ----------------------------------------------------"
		echo
		systemctl is-enabled ntp.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			systemctl enable ntp.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure NTP service ------------------------------------------------"
		echo

		if [ ! -f /etc/ntp.conf.suse ]
		then
			cp /etc/ntp.conf /etc/ntp.conf.suse ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi
		sed -i -e '/^server/d' /etc/ntp.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		if [ ! -z "$NTP_FORWARDER_LIST" ]
		then
			echo -e "$NTP_FORWARDER_LIST" > /etc/ntp.conf
			# sed -i -e 's/^/server '$NTP_FORWARDER'/' /etc/ntp.conf
		fi
		if [ $LOGGING -eq 1 ]; then grep "server" /etc/ntp.conf >&4; fi
		if [ $LOGGING -eq 1 ]; then echo >&4; fi

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting NTP service -------------------------------------------------"
		echo
		systemctl is-enabled ntp.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl stop ntp.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		if [ ! -z "$NTP_FORWARDER_LIST" ]
		then
			ntpdate $NTP_FORWARDER
			if [ $? -ne 0 ]
			then
				echo
				echo "Could not reach ntp $NTP_FORWARDER_LIST"
				echo
				echo "Please check your:"
				echo
				echo "- ntp configuration"
				echo "- DNS resolving"
				echo "- internet connectivity"
				echo

				error_exit "$LINENO"
			fi
		fi
		echo
		systemctl start ntp.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status ntp.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable NTP service ----------------------------------------------------"

		echo
		systemctl is-enabled ntp.service >/dev/null 2>&1 ; RETVAL=$?
		ech0 >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable ntp.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop NTP service -------------------------------------------------------"
		echo
		systemctl is-active ntp.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl stop ntp.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			systemctl status ntp.service
			echo
		else
			RETVAL=0
		fi
	fi

	return $RETVAL

}

# ######### Enable and reconfigure DNS / named service #########
#
reconfigure_dns_service(){

	local RETVAL=0
	local DNS_FORWARDER_ARRAY=( $DNS_FORWARDER )
	local DNS_FORWARDER_LIST

	typeset -i i=0 max=${#DNS_FORWARDER_ARRAY[*]}
	while (( i < max ))
	do
		DNS_FORWARDER_LIST="$DNS_FORWARDER_LIST${DNS_FORWARDER_ARRAY[$i]}; "
		i=i+1
	done

	if [ "$DNS_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Enable DNS / named service ---------------------------------------------"
		echo

		systemctl is-enabled named.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			systemctl enable named.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure DNS / named service ----------------------------------------"
		echo

		if [ ! -f /etc/named.conf.suse ]
		then
			cp /etc/named.conf /etc/named.conf.suse ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		sed -i -e 's/#forwarders /forwarders /' /etc/named.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e 's/#forward /forward /' /etc/named.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^.*forwarders { /s/ { .*};/ { '"$DNS_FORWARDER_LIST"'};/' /etc/named.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^.*forward /s/forward .*;/forward only;/' /etc/named.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		if [ $LOGGING -eq 1 ]; then grep "forward" /etc/named.conf >&4; fi

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting DNS / named service -----------------------------------------"
		echo
		systemctl is-enabled named.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo >/dev/null 2>&1

		### SUSE bug - bind working directory is not writable
		#
		chown named /var/lib/named ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		chgrp named /var/lib/named ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		systemctl restart named.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status named.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable DNS / named service --------------------------------------------"

		echo
		systemctl is-enabled named.service >/dev/null 2>&1 ; RETVAL=$?
		ech0 >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable named.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop DNS / named service -----------------------------------------------"
		echo
		systemctl is-active named.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl stop named.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			systemctl status named.service
			echo
		else
			RETVAL=0
		fi
	fi

	return $RETVAL

}

# ######### Enable and reconfigure WebYaST service #########
#
reconfigure_webyast_service(){

	local RETVAL=0

	if [ "$WEBYAST_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Enable WebYaST service -------------------------------------------------"
		echo
		systemctl is-enabled webyast.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			systemctl enable webyast.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure WebYaST service --------------------------------------------"
		echo

		# ######### Config file /etc/webyast/nginx.conf #########
		#

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting WebYaST service ---------------------------------------------"
		echo
		systemctl is-enabled webyast.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo >/dev/null 2>&1
		systemctl restart webyast.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status webyast.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable WebYaST service ------------------------------------------------"
		echo
		systemctl is-enabled webyast.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable webyast.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop WebYaST service ---------------------------------------------------"
		echo
		systemctl is-active webyast.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl stop webyast.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			systemctl status webyast.service
			echo
		else
			RETVAL=0
		fi
	fi

	return $RETVAL

}


# #########################################################################################
#
# ELMA configure and reconfigure functions
#
# #########################################################################################

# ######### Reconfigure ELMA services during first boot#########
#
reconfigure_firstboot(){

	local RETVAL=0

	# ######### Reconfigure system distribution repositories #########
	#
	reconfigure_update_repositories ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Network service #########
	#
	reconfigure_network_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure SuSEfirewall service #########
	#
	reconfigure_susefirewall_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure openSUSE proxy configuration #########
	#
	###reconfigure_proxy_configuration ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure NTP service #########
	#
	###reconfigure_ntp_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure DNS service #########
	#
	###reconfigure_dns_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure FTP service #########
	#
	###reconfigure_ftp_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure postfix service #########
	#
	###reconfigure_postfix_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure snmpd service #########
	#
	###reconfigure_snmpd_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure snmptrapd service #########
	#
	###reconfigure_snmptrapd_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure webyast service #########
	#
	reconfigure_webyast_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure collectd service #########
	#
	###reconfigure_collectd_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Nagios Core #########
	#
	###reconfigure_nagios_core_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Kibana3 #########
	#
	reconfigure_kibana3 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Apache2 service #########
	#
	reconfigure_apache_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Logstash service #########
	#
	###reconfigure_logstash_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure rsyslog service #########
	#
	reconfigure_rsyslog_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure ElasticSearch service #########
	#
	reconfigure_elasticsearch_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure systemd service #########
	#
	# no reconfigure tasks necessary
	#
	###reconfigure_systemd_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}

# ######### Reconfigure ELMA services #########
#
reconfigure_elma_services(){

	local RETVAL=0

	# ######### Reconfigure system distribution repositories #########
	#
	reconfigure_update_repositories ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Network service #########
	#
	reconfigure_network_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure SuSEfirewall service #########
	#
	reconfigure_susefirewall_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure openSUSE proxy configuration #########
	#
	reconfigure_proxy_configuration ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure NTP service #########
	#
	reconfigure_ntp_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure DNS service #########
	#
	reconfigure_dns_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure FTP service #########
	#
	reconfigure_ftp_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure postfix service #########
	#
	reconfigure_postfix_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure snmpd service #########
	#
	reconfigure_snmpd_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure snmptrapd service #########
	#
	reconfigure_snmptrapd_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure webyast service #########
	#
	reconfigure_webyast_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure collectd service #########
	#
	reconfigure_collectd_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Nagios Core #########
	#
	reconfigure_nagios_core_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Kibana3 #########
	#
	reconfigure_kibana3 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Apache2 service #########
	#
	reconfigure_apache_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure Logstash service #########
	#
	###reconfigure_logstash_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure rsyslog service #########
	#
	reconfigure_rsyslog_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure ElasticSearch service #########
	#
	reconfigure_elasticsearch_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	# ######### Reconfigure systemd service #########
	#
	# no reconfigure tasks necessary
	#
	### reconfigure_systemd_service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	return $RETVAL

}

# ######### Reconfigure rsyslog #########
#
reconfigure_rsyslog_service(){

	local RETVAL=0

	echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure rsyslog ----------------------------------------------------"
	echo

	if [ ! -f $SYSCONFIG_SYSLOG.suse ]
	then
		cp $SYSCONFIG_SYSLOG $SYSCONFIG_SYSLOG.suse ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	sed -i 's/SYSLOG_REQUIRES_NETWORK=""/SYSLOG_REQUIRES_NETWORK="yes"/' $SYSCONFIG_SYSLOG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	if [ $LOGGING -eq 1 ]; then grep "SYSLOG_REQUIRES_NETWORK" $SYSCONFIG_SYSLOG >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	if [ -f $RSYSLOG_MAIN_QUEUE_INCLUDE_CONFIG ]  && [ "$RSYSLOG_MAIN_QUEUE_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_MAIN_QUEUE_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_MAIN_QUEUE_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_DROP_FILTER_INCLUDE_CONFIG ]  && [ "$RSYSLOG_DROP_FILTER_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_DROP_FILTER_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|#.*call drop|s|#||' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_DROP_FILTER_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|^[^#].*call drop|s|^|#|' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_LIBLOGNORM_NORMALIZATION_INCLUDE_CONFIG ]  && [ "$RSYSLOG_LIBLOGNORM_NORMALIZATION_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_LIBLOGNORM_NORMALIZATION_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_LIBLOGNORM_NORMALIZATION_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_CEE_NORMALIZATION_INCLUDE_CONFIG ]  && [ "$RSYSLOG_CEE_NORMALIZATION_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_CEE_NORMALIZATION_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_CEE_NORMALIZATION_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_JSON_FORMAT_INCLUDE_CONFIG ]  && [ "$RSYSLOG_JSON_FORMAT_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_JSON_FORMAT_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_JSON_FORMAT_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_FORWARDING_ES_INCLUDE_CONFIG ]  && [ "$RSYSLOG_FORWARDING_ES_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_FORWARDING_ES_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|#.*call elasticsearch|s|#||' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|#.*call elasticsearchnagios|s|#||' $RSYSLOG_NAGIOS_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_FORWARDING_ES_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|^[^#].*call elasticsearch|s|^|#|' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|^[^#].*call elasticsearchnagios|s|^|#|' $RSYSLOG_NAGIOS_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_LOCAL_LOGFILE_INCLUDE_CONFIG ]  && [ "$RSYSLOG_LOCAL_LOGFILE_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_LOCAL_LOGFILE_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|#.*call logfile|s|#||' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_LOCAL_LOGFILE_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|^[^#].*call logfile|s|^|#|' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_FORWARDING_REMOTE_SYSLOG_INCLUDE_CONFIG ]  && [ "$RSYSLOG_FORWARDING_REMOTE_SYSLOG_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_FORWARDING_REMOTE_SYSLOG_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|#.*call forward|s|#||' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_FORWARDING_REMOTE_SYSLOG_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|^[^#].*call forward|s|^|#|' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_FORWARDING_REMOTE_TLS_RSYSLOG_INCLUDE_CONFIG ]  && [ "$RSYSLOG_FORWARDING_REMOTE_TLS_RSYSLOG_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_FORWARDING_REMOTE_TLS_RSYSLOG_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|#.*call encryptedforwarding|s|#||' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_FORWARDING_REMOTE_TLS_RSYSLOG_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|^[^#].*call encryptedforwarding|s|^|#|' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_MAIL_ALERTING_INCLUDE_CONFIG ]  && [ "$RSYSLOG_MAIL_ALERTING_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_MAIL_ALERTING_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|#.*call mailalert|s|#||' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_MAIL_ALERTING_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|^[^#].*call mailalert|s|^|#|' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_CUSTOMIZE_ACTION_TEMPLATE_INCLUDE_CONFIG ]  && [ "$RSYSLOG_CUSTOMIZE_ACTION_TEMPLATE_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_CUSTOMIZE_ACTION_TEMPLATE_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|#.*call customizedtemplate|s|#||' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_CUSTOMIZE_ACTION_TEMPLATE_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|^[^#].*call customizedtemplate|s|^|#|' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_LOCAL_SYSLOG_INCLUDE_CONFIG ]  && [ "$RSYSLOG_LOCAL_SYSLOG_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_LOCAL_SYSLOG_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|#.*call local|s|#||' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_LOCAL_SYSLOG_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '\|^[^#].*call local|s|^|#|' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ]  && [ "$RSYSLOG_DEFAULT_RULESET_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_TLS_RULESET_INCLUDE_CONFIG ]  && [ "$RSYSLOG_TLS_RULESET_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_TLS_RULESET_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_TLS_RULESET_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ -f $RSYSLOG_NAGIOS_RULESET_INCLUDE_CONFIG ]  && [ "$RSYSLOG_NAGIOS_RULESET_ENABLED" == "Yes" ]
	then
		sed -i -e '\|#$IncludeConfig '"$RSYSLOG_NAGIOS_RULESET_INCLUDE_CONFIG"'|s|#||' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|^$IncludeConfig '"$RSYSLOG_NAGIOS_RULESET_INCLUDE_CONFIG"'|s|^|#|' $RSYSLOG_CONF_DIR/remote.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	# ######### Include config files, that ELMA distribution provided #########
	#

	sed -i -e '\|$IncludeConfig /etc/rsyslog.d/.*|s|$IncludeConfig /etc/rsyslog.d/.*|$IncludeConfig /etc/rsyslog.d/remote.conf|' $RSYSLOG_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ $LOGGING -eq 1 ]; then grep "RepeatedMsgReduction" $RSYSLOG_CONF >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi
    
	# ######### Reduce dupplicate log messages (last message repeated n times) #########
	#
	if [ "$RSYSLOG_REDUCE_DUPPLICATE_ENABLED" == "Yes" ]
	then
		sed -i -e '\|$RepeatedMsgReduction.*|s|$RepeatedMsgReduction.*|$RepeatedMsgReduction on|' $RSYSLOG_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	else
		sed -i -e '\|$RepeatedMsgReduction.*|s|$RepeatedMsgReduction.*|$RepeatedMsgReduction off|' $RSYSLOG_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	if [ $LOGGING -eq 1 ]; then grep "RepeatedMsgReduction" $RSYSLOG_CONF >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	# ######### Predefined JSON format values #########
	#
	sed -i -e '\|^.*set $!country_code = "|s|country_code = .*|country_code = "'"$RSYSLOG_JSON_FORMAT_COUNTRY_CODE"'";|' $RSYSLOG_JSON_FORMAT_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	sed -i -e '\|^.*set $!customer = "|s|customer = .*|customer = "'"$RSYSLOG_JSON_FORMAT_CUSTOMER"'";|' $RSYSLOG_JSON_FORMAT_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ $LOGGING -eq 1 ]; then grep "country_code" $RSYSLOG_JSON_FORMAT_INCLUDE_CONFIG >&4; fi
	if [ $LOGGING -eq 1 ]; then grep "customer" $RSYSLOG_JSON_FORMAT_INCLUDE_CONFIG >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	# ######### ElasticSearch connection values #########
	#
	sed -i -e '\|^.*server="|s|server=".*|server="'"$RSYSLOG_FORWARDING_ES_SERVER"'"|' $RSYSLOG_FORWARDING_ES_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	sed -i -e '\|^.*serverport="|s|serverport=".*|serverport="'"$RSYSLOG_FORWARDING_ES_PORT"'"|' $RSYSLOG_FORWARDING_ES_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ $LOGGING -eq 1 ]; then grep "server=" $RSYSLOG_FORWARDING_ES_INCLUDE_CONFIG >&4; fi
	if [ $LOGGING -eq 1 ]; then grep "serverport=" $RSYSLOG_FORWARDING_ES_INCLUDE_CONFIG >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	# ######### TLS rsyslog connection values #########
	#
	sed -i -e '\|^.*@@(o)|s|@@(o).*|@@(o)'"$RSYSLOG_FORWARDING_REMOTE_TLS_RSYSLOG_SERVER"':'"$RSYSLOG_FORWARDING_REMOTE_TLS_RSYSLOG_PORT"'|' $RSYSLOG_FORWARDING_REMOTE_TLS_RSYSLOG_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ $LOGGING -eq 1 ]; then grep "@@(o)" $RSYSLOG_FORWARDING_REMOTE_TLS_RSYSLOG_INCLUDE_CONFIG >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	# ######### Local logfile name template #########
	#

	sed -i -e '\|^.*$template DailyPerHostLogs,"|s|$template DailyPerHostLogs,.*|$template DailyPerHostLogs,"'"$RSYSLOG_LOCAL_LOGFILE_DIR"'/'"$RSYSLOG_LOCAL_LOGFILE_NAME"'";|' $RSYSLOG_LOCAL_LOGFILE_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ $LOGGING -eq 1 ]; then grep "DailyPerHostLogs" $RSYSLOG_LOCAL_LOGFILE_INCLUDE_CONFIG >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	# ######### Update rsyslog listener configuration for default ruleset #########
	#
	sed -i -e '\|^.*ruleBase="|s|ruleBase=.*|ruleBase="'"$RSYSLOG_LIBLOGNORM_NORMALIZATION_RULEBASE_FILE"'"|' $RSYSLOG_LIBLOGNORM_NORMALIZATION_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	sed -i -e '\|^.*useRawMsg="|s|useRawMsg=.*|useRawMsg="'"$RSYSLOG_LIBLOGNORM_NORMALIZATION_RULEBASE_USERAWMSG"'"|' $RSYSLOG_LIBLOGNORM_NORMALIZATION_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ $LOGGING -eq 1 ]; then grep "input" $RSYSLOG_LIBLOGNORM_NORMALIZATION_INCLUDE_CONFIG >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	# ######### Update rsyslog listener configuration for default ruleset #########
	#
	sed -i -e '/^.*input(type="imtcp" port="/s/port=.*/port="'"$RSYSLOG_DEFAULT_RULESET_TCP_PORT"'" ruleset="default")/' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	sed -i -e '/^.*input(type="imudp" port="/s/port=.*/port="'"$RSYSLOG_DEFAULT_RULESET_UDP_PORT"'" ruleset="default")/' $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ $LOGGING -eq 1 ]; then grep "input" $RSYSLOG_DEFAULT_RULESET_INCLUDE_CONFIG >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	# ######### Update rsyslog listener configuration for TLS ruleset #########
	#
	sed -i -e '/^.*input(type="imtcp" port="/s/port=.*/port="'"$RSYSLOG_TLS_RULESET_TCP_PORT"'" ruleset="default")/' $RSYSLOG_TLS_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ $LOGGING -eq 1 ]; then grep "input" $RSYSLOG_TLS_RULESET_INCLUDE_CONFIG >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	# ######### Update rsyslog listener configuration for Nagios ruleset #########
	#
	sed -i -e '/^.*input(type="imtcp" port="/s/port=.*/port="'"$RSYSLOG_NAGIOS_RULESET_TCP_PORT"'" ruleset="nagios")/' $RSYSLOG_NAGIOS_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	sed -i -e '/^.*input(type="imudp" port="/s/port=.*/port="'"$RSYSLOG_NAGIOS_RULESET_UDP_PORT"'" ruleset="nagios")/' $RSYSLOG_NAGIOS_RULESET_INCLUDE_CONFIG ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ $LOGGING -eq 1 ]; then grep "input" $RSYSLOG_NAGIOS_RULESET_INCLUDE_CONFIG >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	# ######### Set ElasticSearch service UID password in rsyslog config files #########
	#
	sed -i -e '/^.*uid=/s/"[^*]*"/"'$ES_UID'"/' $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	sed -i -e '/^.*pwd=/s/"[^*]*"/"'$ES_PWD'"/' $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

	if [ $LOGGING -eq 1 ]; then grep "uid=" $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf >&4; fi
	if [ $LOGGING -eq 1 ]; then grep "pwd=" $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	# ######### Enable/Disable ElasticSearch basic authentication #########
	#
	if [ $LOGGING -eq 1 ]; then echo "--- `date +%T.%N | cut -c 1-12` - Enable/Disable ElasticSearch basic authentication ----------------------" >&4; fi
	if [ $LOGGING -eq 1 ]; then echo >&4; fi

	if [ "$ES_AUTH" == "" ]; then

		# Disable ElasticSearch basic authentication
		if [ $LOGGING -eq 1 ]; then echo "Disable ElasticSearch basic authentication" >&4; fi
		if [ $LOGGING -eq 1 ]; then echo >&4; fi

		# Disable rsyslog basic authentication in all configuration templates
		if [ $LOGGING -eq 1 ]; then echo "Disable rsyslog basic authentication in all configuration templates" >&4; fi
		if [ $LOGGING -eq 1 ]; then echo >&4; fi

		sed -i -e '/^[^#].*uid=/s/^/#/' $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		sed -i -e '/^[^#].*pwd=/s/^/#/' $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		if [ $LOGGING -eq 1 ]; then grep "uid=" $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf >&4; fi
		if [ $LOGGING -eq 1 ]; then grep "pwd=" $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf >&4; fi
		if [ $LOGGING -eq 1 ]; then echo >&4; fi

	else

		# Enable ElasticSearch basic authentication
		if [ $LOGGING -eq 1 ]; then echo "Enable ElasticSearch basic authentication" >&4; fi
		if [ $LOGGING -eq 1 ]; then echo >&4; fi

		RSYSLOG_ES_HOST=$(grep 'server="localhost"' $RSYSLOG_CONF_REMOTE | awk -F'"' '{print $2}')

		if [ ! "$RSYSLOG_ES_HOST" == "localhost" ]; then
			# Enable rsyslog basic authentication
			if [ $LOGGING -eq 1 ]; then echo "Enable rsyslog basic authentication" >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi

			sed -i -e '/#.*uid=/s/#//' $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/#.*pwd=/s/#//' $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			if [ $LOGGING -eq 1 ]; then grep "uid=" $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "pwd=" $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi
		else
			# Disable rsyslog basic authentication for ElasticSearch on localhost
			if [ $LOGGING -eq 1 ]; then echo "Disable rsyslog basic authentication for ElasticSearch on localhost" >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi

			sed -i -e '/^[^#].*uid=/s/^/#/' $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/^[^#].*pwd=/s/^/#/' $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			if [ $LOGGING -eq 1 ]; then grep "uid=" $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "pwd=" $RSYSLOG_CONF_DIR/elma_output-elasticsearch-action-template.conf >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi
		fi
	fi

	echo "--- `date +%T.%N | cut -c 1-12` - Restarting rsyslog -----------------------------------------------------"
	echo
	systemctl is-enabled syslog.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	echo >/dev/null 2>&1
	systemctl restart syslog.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	systemctl status syslog.service
	echo

	return $RETVAL

}

# ######### Enable and reconfigure Logstash service #########
#
reconfigure_logstash_service(){

	local RETVAL=0

	if [ "$LOGSTASH_ENABLED" = "Yes" ]
	then

		echo "--- `date +%T.%N | cut -c 1-12` - Update Logstash service ------------------------------------------------"
		echo

		systemctl --system daemon-reload ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		echo "--- `date +%T.%N | cut -c 1-12` - Enable Logstash service ------------------------------------------------"
		echo
		systemctl is-enabled logstash.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			systemctl enable logstash.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Logstash service -------------------------------------------"
		echo

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting Logstash service --------------------------------------------"
		echo
		systemctl is-enabled logstash.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo >/dev/null 2>&1
		systemctl restart logstash.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status logstash.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable Logstash service -----------------------------------------------"
		echo
		systemctl is-enabled logstash.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1 ; 
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable logstash.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop Logstash service --------------------------------------------------"
		echo
		systemctl is-active logstash.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1

		if [ $RETVAL -eq 0 ]
		then
			systemctl stop logstash.service; RETVAL=$?
			systemctl status logstash.service
			echo
		else
			RETVAL=0
		fi
	fi

	return $RETVAL

}

# ######### Enable and reconfigure ElasticSearch service #########
#
reconfigure_elasticsearch_service(){

	local RETVAL=0

	if [ "$(systemctl list-unit-files | grep elasticsearch.service)" ]
	then
		if [ "$ES_ENABLED" = "Yes" ]
		then
			echo "--- `date +%T.%N | cut -c 1-12` - Enable ElasticSearch service -------------------------------------------"
			echo
			systemctl is-enabled elasticsearch.service >/dev/null 2>&1 ; RETVAL=$?
			echo >/dev/null 2>&1
			if [ $RETVAL -ne 0 ]
			then
				systemctl enable elasticsearch.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				echo
				RETVAL=0
			fi

			echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure ElasticSearch service --------------------------------------"
			echo

			# ######### Set ElasticSearch heap space #########
			#
			if [ -f $SYSCONFIG_DIR/elasticsearch ]
			then
				sed -i -e '/^ES_HEAP_SIZE=/s|=.*|='$ES_HEAP_SIZE'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi

			if [ ! -d $ES_DATA_DIR ]
			then
				mkdir $ES_DATA_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi
			chown -R elasticsearch $ES_DATA_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			chgrp -R elasticsearch $ES_DATA_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			if [ -f $SYSCONFIG_DIR/elasticsearch ]
			then
				sed -i -e '/^ES_CLUSTER_NAME=/s|=.*|='$ES_CLUSTER_NAME'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^ES_NODE_NAME=/s|=.*|='$HOST_FQDN'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^ES_HEAP_SIZE=/s|=.*|='$ES_HEAP_SIZE'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^ES_HTTP_HOST=/s|=.*|='localhost'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^ES_HOME=/s|=.*|='$ES_DIR'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^ES_PLUGIN_DIR=/s|=.*|='$ES_DIR'/plugins|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^LOG_DIR=/s|=.*|='$ES_LOG_DIR'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^LOCK_DIR=/s|=.*|='$ES_LOCK_DIR'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^DATA_DIR=/s|=.*|='$ES_DATA_DIR'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^WORK_DIR=/s|=.*|='$ES_WORK_DIR'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^CONF_DIR=/s|=.*|='$ES_CONF_DIR'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^CONF_FILE=/s|=.*|='$ES_CONF'|' $SYSCONFIG_DIR/elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi

			# ######### Set ElasticSearch service UID password #########
			#
			htpasswd2 -bs $APACHE2_HTPASSWD_FILE $ES_UID $ES_PWD >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			echo "--- `date +%T.%N | cut -c 1-12` - Restarting ElasticSearch service ---------------------------------------"
			echo
			systemctl is-enabled elasticsearch.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo >/dev/null 2>&1
			systemctl restart elasticsearch.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sleep 15
			systemctl status elasticsearch.service
			echo
		else
			echo "--- `date +%T.%N | cut -c 1-12` - Disable ElasticSearch service ------------------------------------------"
			echo
			systemctl is-enabled elasticsearch.service >/dev/null 2>&1 ; RETVAL=$?
			echo >/dev/null 2>&1
			if [ $RETVAL -eq 0 ]
			then
				systemctl disable elasticsearch.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				echo
			else
				RETVAL=0
			fi

			echo "--- `date +%T.%N | cut -c 1-12` - Stop ElasticSearch service ---------------------------------------------"
			echo
			systemctl is-active elasticsearch.service >/dev/null 2>&1 ; RETVAL=$?
			echo >/dev/null 2>&1
			if [ $RETVAL -eq 0 ]
			then
				systemctl stop elasticsearch.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sleep 5
				systemctl status elasticsearch.service
				echo
			else
				RETVAL=0
			fi
		fi
	fi

	return $RETVAL

}

# ######### Reconfigure Kibana3 #########
#
reconfigure_kibana3(){

	local RETVAL=0

	if [ "$KIBANA3_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Kibana3 ----------------------------------------------------"
		echo
		if [ -f $KIBANA3_CONF ]
		then
			sed -i 's/"http:\/\/"+window.location.hostname+":9200",/"https:\/\/"+window.location.host+"\/elasticsearch\/",/' $KIBANA3_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			if [ $LOGGING -eq 1 ]; then grep "elasticsearch:" $KIBANA3_CONF >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi
		fi
	fi

	return $RETVAL

}

# ######### Enable and reconfigure Nagios Core service #########
#
reconfigure_nagios_core_service(){

	local RETVAL=0

	if [ "$NAGIOS_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Enable Nagios Core service ---------------------------------------------"
		echo
		systemctl is-enabled nagios.service >/dev/null 2>&1 ; RETVAL=$?
		echo  >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			systemctl enable nagios.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
			RETVAL=0
		fi

	# ######### Customize Nagios Core #########
	#
	echo "--- `date +%T.%N | cut -c 1-12` - Customize Nagios Core --------------------------------------------------"
	echo

#	# Customize original Nagios Core index.php
#	#
#	if [ -f $NAGIOS_DIR/index.php ]
#	then
#		sed -i -e 's/$corewindow=.*/$corewindow="cgi-bin\/tac.cgi";/' $NAGIOS_DIR/index.php ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#		sed -i -e 's/<frameset cols=.*/<frameset cols="200,*" style="border: 0px; framespacing: 0px">/' $NAGIOS_DIR/index.php ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	fi
#
#	if [ -f $NAGIOS_THEME_DIR/index.php ]
#	then
#		sed -i -e 's/$corewindow=.*/$corewindow="cgi-bin\/tac.cgi";/' $NAGIOS_THEME_DIR/index.php ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#		sed -i -e 's/<frameset cols=.*/<frameset cols="200,*" style="border: 0px; framespacing: 0px">/' $NAGIOS_THEME_DIR/index.php ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	fi
#
#	# Customize original Nagios Core side.php
#	#
#	if [ -f $NAGIOS_DIR/side.php ] && [ ! -f $NAGIOS_DIR/side.php.bak ]
#	then
#		sed -i.bak -e 's|Trends</a></li>|Trends</a></li>\n<li><a href="<?php echo $cfg["cgi_base_url"];?>/show.cgi" target="<?php echo $link_target;?>">Graphs</a>\n<ul>\n<li><a href="<?php echo $cfg["cgi_base_url"];?>/showhost.cgi" target="<?php echo $link_target;?>">Graphs by Host</a></li>\n<li><a href="<?php echo $cfg["cgi_base_url"];?>/showservice.cgi" target="<?php echo $link_target;?>">Graphs by Service</a></li>\n<li><a href="<?php echo $cfg["cgi_base_url"];?>/showgroup.cgi" target="<?php echo $link_target;?>">Graphs by Group</a></li>\n</ul>\n</li>\n|' $NAGIOS_DIR/side.php ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	fi
#
#	if [ -f $NAGIOS_THEME_DIR/side.php ] && [ ! -f $NAGIOS_THEME_DIR/side.php.bak ]
#	then
#		sed -i.bak -e 's|Trends</a></li>|Trends</a></li>\n<li><a href="<?php echo $cfg["cgi_base_url"];?>/show.cgi" target="<?php echo $link_target;?>">Graphs</a>\n<ul>\n<li><a href="<?php echo $cfg["cgi_base_url"];?>/showhost.cgi" target="<?php echo $link_target;?>">Graphs by Host</a></li>\n<li><a href="<?php echo $cfg["cgi_base_url"];?>/showservice.cgi" target="<?php echo $link_target;?>">Graphs by Service</a></li>\n<li><a href="<?php echo $cfg["cgi_base_url"];?>/showgroup.cgi" target="<?php echo $link_target;?>">Graphs by Group</a></li>\n</ul>\n</li>\n|' $NAGIOS_THEME_DIR/side.php ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	fi

	# Reconfigure Apache2 Nagios service
	#
	reconfigure_apache_nagios ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

#	# Replace original Nagios Core stylesheets with Exfoliation a simple makeover for the Nagios Core web interface
#	#
#	if [ -f $USR_SBIN_DIR/switch-nagios-theme ] && [ -d $NAGIOS_THEME_DIR ]
#	then
#		$USR_SBIN_DIR/switch-nagios-theme  exfoliation  ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#	fi

		# ######### Configuring Nagios Core #########
		#
		echo "--- `date +%T.%N | cut -c 1-12` - Configuring Nagios Core ------------------------------------------------"
		echo

		if [ -f $NAGIOS_CONF_DIR/elma/hosts.cfg ]
		then
			sed -i -e 's/^ name .*/ name '$HOST_FQDN'/g' $NAGIOS_CONF_DIR/elma/hosts.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/^ host_name .*/ host_name '$HOST_FQDN'/g' $NAGIOS_CONF_DIR/elma/hosts.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/^ alias .*/ alias '$HOST_FQDN'/g' $NAGIOS_CONF_DIR/elma/hosts.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/^ address 127.0.0.1.*/ address '$HOST_IP'/g' $NAGIOS_CONF_DIR/elma/hosts.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		if [ -f $NAGIOS_CONF_DIR/nagios.cfg ]
		then
			if [ ! -f $NAGIOS_CONF_DIR/nagios.cfg.suse ]
			then
				cp $NAGIOS_CONF_DIR/nagios.cfg  $NAGIOS_CONF_DIR/nagios.cfg.suse ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi
			sed -i -e 's/^cfg_file=/#cfg_file=/g' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			sed -i -e '/service_check_timeout=/s/=.*/=90/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/host_check_timeout=/s/=.*/=90/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/event_handler_timeout=/s/=.*/=90/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/notification_timeout=/s/=.*/=90/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			sed -i -e '/process_performance_data=/s/=.*/=1/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#host_perfdata_command=.*/host_perfdata_command=process-host-perfdata/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#service_perfdata_command=.*/service_perfdata_command=process-service-perfdata/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's|#host_perfdata_file=.*|host_perfdata_file=/var/log/nagios/host-perfdata|' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's|#service_perfdata_file.*=|service_perfdata_file=/var/log/nagios/service-perfdata|' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#host_perfdata_file_template=.*/host_perfdata_file_template=$LASTSERVICECHECK$||$HOSTNAME$||$SERVICEDESC$||$SERVICEOUTPUT$||$SERVICEPERFDATA$/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#service_perfdata_file_template=.*/service_perfdata_file_template=$LASTSERVICECHECK$||$HOSTNAME$||$SERVICEDESC$||$SERVICEOUTPUT$||$SERVICEPERFDATA$/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#host_perfdata_file_mode=/host_perfdata_file_mode=/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#service_perfdata_file_mode=/service_perfdata_file_mode=/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#host_perfdata_file_processing_interval=/host_perfdata_file_processing_interval=/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#service_perfdata_file_processing_interval=/service_perfdata_file_processing_interval=/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#host_perfdata_file_processing_command=.*/#host_perfdata_file_processing_command=process-host-perfdata/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#service_perfdata_file_processing_command=.*/#service_perfdata_file_processing_command=process-service-perfdata/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#host_perfdata_process_empty_results=/host_perfdata_process_empty_results=/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/#service_perfdata_process_empty_results=/service_perfdata_process_empty_results=/' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			grep "cfg_dir=/etc/nagios/elma" $NAGIOS_CONF_DIR/nagios.cfg >/dev/null 2>&1 ; RETVAL=$?
			echo >/dev/null 2>&1
			if [ $RETVAL == 1 ]
			then
				sed -i -e '/cfg_dir=\/etc\/nagios\/routers/a\
cfg_dir=\/etc\/nagios\/elma
' $NAGIOS_CONF_DIR/nagios.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				RETVAL=$?
			fi
		fi

		if [ -f $NAGIOS_CONF_DIR/cgi.cfg ]
		then
			if [ ! -f $NAGIOS_CONF_DIR/cgi.cfg.suse ]
			then
				cp $NAGIOS_CONF_DIR/cgi.cfg $NAGIOS_CONF_DIR/cgi.cfg.suse ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi
			sed -i -e 's/physical_html_path=.*/physical_html_path=\/usr\/share\/nagios/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/authorized_for_system_information=.*/authorized_for_system_information=*/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/authorized_for_configuration_information=.*/authorized_for_configuration_information=*/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/authorized_for_system_commands=.*/authorized_for_system_commands='$NAGIOS_ADMIN_UID'/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/authorized_for_all_services=.*/authorized_for_all_services=*/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/authorized_for_all_hosts=.*/authorized_for_all_hosts=*/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/authorized_for_all_service_commands=.*/authorized_for_all_service_commands=*/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/authorized_for_all_host_commands=.*/authorized_for_all_host_commands=*/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/action_url_target=.*/action_url_target=main/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/notes_url_target=.*/notes_url_target=main/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		# ######### Configuring Nagiosgraph #########
		#
		echo "--- `date +%T.%N | cut -c 1-12` - Configuring Nagiosgraph ------------------------------------------------"
		echo

		if [ -f $NAGIOSGRAPH_CONF_DIR/access.conf ]
		then
			if [ ! -f $NAGIOSGRAPH_CONF_DIR/access.conf.suse ]
			then
				cp $NAGIOSGRAPH_CONF_DIR/access.conf $NAGIOSGRAPH_CONF_DIR/access.conf.suse ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi
			sed -i -e 's/^*=/*,*=*/' $NAGIOSGRAPH_CONF_DIR/access.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		if [ -f $NAGIOSGRAPH_CONF_DIR/datasetdb.conf ]
		then
			if [ ! -f $NAGIOSGRAPH_CONF_DIR/datasetdb.conf.suse ]
			then
				cp $NAGIOSGRAPH_CONF_DIR/datasetdb.conf $NAGIOSGRAPH_CONF_DIR/datasetdb.conf.suse ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi
			sed -i -e 's/^service=/# service=/' $NAGIOSGRAPH_CONF_DIR/datasetdb.conf ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		if [ -f $NAGIOSGRAPH_CONF_DIR/groupdb.conf ]
		then
			if [ ! -f $NAGIOSGRAPH_CONF_DIR/groupdb.conf.suse ]
			then
				mv $NAGIOSGRAPH_CONF_DIR/groupdb.conf $NAGIOSGRAPH_CONF_DIR/groupdb.conf.suse ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi
		fi

		if [ -f $NAGIOSGRAPH_CONF_DIR/hostdb.conf ]
		then
			if [ ! -f $NAGIOSGRAPH_CONF_DIR/hostdb.conf.suse ]
			then
				cp $NAGIOSGRAPH_CONF_DIR/hostdb.conf $NAGIOSGRAPH_CONF_DIR/hostdb.conf.suse ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Nagios Core service ----------------------------------------"
		echo

		if [ -f $NAGIOS_CONF_DIR/cgi.cfg ]
		then
			sed -i -e 's/authorized_for_system_commands=.*/authorized_for_system_commands='$NAGIOS_ADMIN_UID'/' $NAGIOS_CONF_DIR/cgi.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		if [ -f $NAGIOS_CONF_DIR/elma/contacts.cfg ]
		then
			sed -i -e 's/^ email admin.*/ email '$NAGIOS_ADMIN_EMAIL'/g' $NAGIOS_CONF_DIR/elma/contacts.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/^ pager admin.*/ pager '$NAGIOS_ADMIN_PAGER'/g' $NAGIOS_CONF_DIR/elma/contacts.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e 's/^ members .*/ members '$NAGIOS_ADMIN_UID'/g' $NAGIOS_CONF_DIR/elma/contacts.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		if [ ! -z "$SNMP_READ_ONLY_COMMUNITY" ]
		then
			if [ -f /etc/nagios/resource.cfg ]
			then
				if [ -z "$(grep '$USER6' /etc/nagios/resource.cfg)" ]
				then
					echo -e "\n# SNMP READ ONLY COMMUNITY\n\$USER6\$=$SNMP_READ_ONLY_COMMUNITY" >> /etc/nagios/resource.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				fi

				sed -i -e '/^$USER6$=/s/=.*/='$SNMP_READ_ONLY_COMMUNITY'/' /etc/nagios/resource.cfg ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				if [ $LOGGING -eq 1 ]; then grep "USER6" /etc/nagios/resource.cfg >&4; fi
				if [ $LOGGING -eq 1 ]; then echo >&4; fi
			fi
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting Nagios Core service -----------------------------------------"
		echo
		systemctl is-enabled nagios.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo  >/dev/null 2>&1
		systemctl restart nagios.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status nagios.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable Nagios Core service --------------------------------------------"
		echo
		systemctl is-enabled nagios.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable nagios.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop Nagios Core service -----------------------------------------------"
		echo
		systemctl is-active nagios.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl stop nagios.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			systemctl status nagios.service
			echo
		else
			RETVAL=0
		fi
	fi

	return $RETVAL

}

# ######### Set permissions for ELMA web directory #########
#
reconfigure_elma_web_directory(){

	local RETVAL=0

	if [ "$APACHE2_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Set permissions for ELMA web directory ---------------------------------"
		echo

		chown -R wwwrun $ELMA_WWW_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		chgrp -R www $ELMA_WWW_DIR ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		find $ELMA_WWW_DIR -type d -exec chmod 0755 {} \; ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		find $ELMA_WWW_DIR -type f -exec chmod 0644 {} \; ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
	fi

	return $RETVAL

}

# ######### Reconfigure Apache2 default authentication function #########
#
reconfigure_apache_default_authentication(){

	local RETVAL=0

	if [ "$APACHE2_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Apache2 default authentication -----------------------------"
		echo

		if [ -f $APACHE2_HTPASSWD_FILE ]
		then
			chown wwwrun $APACHE2_HTPASSWD_FILE ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		if [ -f $APACHE2_CONF_DEFAULT_AUTHENTICATION ]
		then
			sed -i -e '/AuthBasicProvider /s/ .*/ '"$APACHE2_AUTH_BASIC_PROVIDER"'/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/AuthUserFile /s|".*"|"'$APACHE2_HTPASSWD_FILE'"|g' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			if [ ! -z "$(echo $APACHE2_AUTH_BASIC_PROVIDER | grep "ldap")" ]
			then
				sed -i -e 's/#.*AuthLDAPBindDN/AuthLDAPBindDN/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e 's/#.*AuthLDAPBindPassword/AuthLDAPBindPassword/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e 's/#.*AuthLDAPURL/AuthLDAPURL/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e 's/#.*AuthLDAPGroupAttributeIsDN/AuthLDAPGroupAttributeIsDN/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

				AuthLDAPBindDN="${AuthLDAPBindDN/&/\&}"
				sed -i -e 's/AuthLDAPBindDN .*/AuthLDAPBindDN "'"$APACHE2_AUTH_LDAP_BIND_DN"'"/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

				AuthLDAPBindPassword="${AuthLDAPBindPassword/&/\&}"
				sed -i -e 's/AuthLDAPBindPassword .*/AuthLDAPBindPassword "'"$APACHE2_AUTH_LDAP_BIND_PASSWORD"'"/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

				APACHE2_AUTH_LDAP_URL="${APACHE2_AUTH_LDAP_URL/&/\&}"
				sed -i -e 's|AuthLDAPURL .*|AuthLDAPURL "'"$APACHE2_AUTH_LDAP_URL"'"|' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			else
				sed -i -e 's/[#].*AuthLDAPBindDN/# AuthLDAPBindDN/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e 's/[#].*AuthLDAPBindPassword/# AuthLDAPBindPassword/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e 's/[#].*AuthLDAPURL/# AuthLDAPURL/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e 's/[#].*AuthLDAPGroupAttributeIsDN/# AuthLDAPGroupAttributeIsDN/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi

			if [ ! -z "$(echo $APACHE2_AUTH_BASIC_PROVIDER | grep "xradius")" ]
			then
				sed -i -e 's/#.*AuthXRadiusAddServer/AuthXRadiusAddServer/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e 's/#.*AuthXRadiusTimeout/AuthXRadiusTimeout/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e 's/#.*AuthXRadiusRetries/AuthXRadiusRetries/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

				local XRADIUS_SERVER_ARRAY=( $APACHE2_AUTH_XRADIUS_SERVER )

				typeset -i i=0 max=${#XRADIUS_SERVER_ARRAY[*]}
				while (( i < max ))
				do

					if [ $i -eq 0 ]
					then
						sed -i -e '/^.*[^#]AuthXRadiusAddServer /s/ .*/ "'${XRADIUS_SERVER_ARRAY[$i]}':'$APACHE2_AUTH_XRADIUS_PORT'" "'$APACHE2_AUTH_XRADIUS_KEY'"/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
					else
						sed -i -e '/^.*[^#]AuthXRadiusAddServer "'${XRADIUS_SERVER_ARRAY[$i-1]}':'$APACHE2_AUTH_XRADIUS_PORT'" "'$APACHE2_AUTH_XRADIUS_KEY'"/a \\tAuthXRadiusAddServer "'${XRADIUS_SERVER_ARRAY[$i]}':'$APACHE2_AUTH_XRADIUS_PORT'" "'$APACHE2_AUTH_XRADIUS_KEY'"' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
					fi

					i=i+1
				done

				sed -i -e '/^.*[^#]AuthXRadiusTimeout /s/ .*/ '"$APACHE2_AUTH_XRADIUS_TIMEOUT"'/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e '/^.*[^#]AuthXRadiusRetries /s/ .*/ '"$APACHE2_AUTH_XRADIUS_RETRIES"'/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			else
				sed -i -e 's/[#].*AuthXRadiusAddServer/# AuthXRadiusAddServer/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e 's/[#].*AuthXRadiusTimeout/# AuthXRadiusTimeout/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				sed -i -e 's/[#].*AuthXRadiusRetries/# AuthXRadiusRetries/' $APACHE2_CONF_DEFAULT_AUTHENTICATION ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			fi

			if [ $LOGGING -eq 1 ]; then grep "AuthBasicProvider" $APACHE2_CONF_DEFAULT_AUTHENTICATION >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "AuthUserFile" $APACHE2_CONF_DEFAULT_AUTHENTICATION >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "AuthLDAP" $APACHE2_CONF_DEFAULT_AUTHENTICATION >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "AuthXRadius" $APACHE2_CONF_DEFAULT_AUTHENTICATION >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi
		fi
	fi

	return $RETVAL

}

# ######### Reconfigure Apache2 http redirect function #########
#
reconfigure_apache_redirect_http(){

	local RETVAL=0

	if [ "$APACHE2_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Apache2 http redirect --------------------------------------"
		echo

		if [ -f $APACHE2_CONF_REDIRECT_HTTP ]
		then
			sed -i -e '/^.*ServerAdmin/s/ServerAdmin.*/ServerAdmin '$ELMA_MAIL_ADDRESS'/' $APACHE2_CONF_REDIRECT_HTTP ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/^.*ServerName/s/ServerName.*/ServerName '$ELMA_WWW_FQDN'/' $APACHE2_CONF_REDIRECT_HTTP ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			if [ $LOGGING -eq 1 ]; then grep "ServerAdmin" $APACHE2_CONF_REDIRECT_HTTP >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "ServerName" $APACHE2_CONF_REDIRECT_HTTP >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi
		fi
	fi

	return $RETVAL

}

# ######### Reconfigure Apache2 ElasticSearch #########
#
reconfigure_apache_elasticsearch(){

	local RETVAL=0
	local ES_RSYSLOG_CENTRAL_SERVER_ARRAY=( $ES_RSYSLOG_CENTRAL_SERVER )

	if [ "$APACHE2_ENABLED" = "Yes" ] && [ "$ES_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Apache2 ElasticSearch --------------------------------------"
		echo

		if [ -f $ES_APACHE_CONF ] && [ -d $ES_DIR ]
		then
			sed -i -e '/^.*ProxyPass \/ /s/\/ .*/\/ '$ES_PROTOCOL':\/\/'$ES_HOST':'$ES_PORT'\//' $ES_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/^.*ProxyPassReverse \/ /s/\/ .*/\/ '$ES_PROTOCOL':\/\/'$ES_HOST':'$ES_PORT'\//' $ES_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			sed -i -e '/^.*<VirtualHost /s/<VirtualHost .*/<VirtualHost *:'$ES_PLUGIN_PORT'>/' $ES_APACHE_CONF; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/^.*ServerAdmin/s/ServerAdmin.*/ServerAdmin '$ELMA_MAIL_ADDRESS'/' $ES_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/^.*ServerName/s/ServerName.*/ServerName '$ELMA_WWW_FQDN'/' $ES_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			if [ $LOGGING -eq 1 ]; then grep "<VirtualHost" $ES_APACHE_CONF >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "ProxyPass" $ES_APACHE_CONF >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "ServerAdmin" $ES_APACHE_CONF >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "ServerName" $ES_APACHE_CONF >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi

			echo "--- `date +%T.%N | cut -c 1-12` - Modify apache2 elasticsearch listener configuration --------------------"
			echo
			if [ -f $APACHE2_CONF_LISTEN ]
			then
				grep "Listen.*$ES_PLUGIN_PORT" $APACHE2_CONF_LISTEN >/dev/null 2>&1 ; RETVAL=$?
				if [ $RETVAL == 1 ]
				then
					sed -i -e '/^.*<IfModule mod_ssl.c>/s/<IfModule mod_ssl.c>/<IfModule mod_ssl.c>\n\n            Listen '$ES_PLUGIN_PORT'/' $APACHE2_CONF_LISTEN ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
					RETVAL=0
				fi

				if [ $LOGGING -eq 1 ]; then grep "Listen" $APACHE2_CONF_LISTEN >&4; fi
				if [ $LOGGING -eq 1 ]; then echo >&4; fi
			fi
		fi

		if [ -f $APACHE2_CONF_LISTEN ]
		then

			grep "Listen.*$ES_PORT" $APACHE2_CONF_LISTEN >&4; RETVAL=$?

			if [ $RETVAL == 0 ]
			then
				sed -i -e '/^.*'$ES_PORT'/d' $APACHE2_CONF_LISTEN ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			else
				RETVAL=0
			fi
		fi
	fi

	return $RETVAL

}

# ######### Reconfigure Apache2 Kibana3 #########
#
reconfigure_apache_kibana3(){

	local RETVAL=0

	if [ "$APACHE2_ENABLED" = "Yes" ] && [ "$KIBANA3_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Apache2 Kibana3 --------------------------------------------"
		echo

		if [ -f $KIBANA3_APACHE_CONF ] && [ -d $KIBANA3_DIR ]
		then

			sed -i -e '/^.*ProxyPass \/elasticsearch\/ /s/\/elasticsearch\/ .*/\/elasticsearch\/ '$ES_PROTOCOL':\/\/'$ES_HOST':'$ES_PORT'\//' $KIBANA3_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/^.*ProxyPassReverse \/elasticsearch\/ /s/\/elasticsearch\/ .*/\/elasticsearch\/ '$ES_PROTOCOL':\/\/'$ES_HOST':'$ES_PORT'\//' $KIBANA3_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			if [ $LOGGING -eq 1 ]; then grep "Alias " $KIBANA3_APACHE_CONF >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "ProxyPass " $KIBANA3_APACHE_CONF >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "<Directory" $KIBANA3_APACHE_CONF >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi
		fi
	fi

	return $RETVAL

}

# ######### Reconfigure Apache2 Nagios function #########
#
reconfigure_apache_nagios(){

	local RETVAL=0

	if [ "$APACHE2_ENABLED" = "Yes" ] && [ "$NAGIOS_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Apache2 Nagios ---------------------------------------------"
		echo

#		if [ -f $NAGIOS_APACHE_CONF ] && [ -d $NAGIOS_THEME_DIR ] && [ -d $NAGIOS_CGI_DIR ]
#		then
#			sed -i -e '/^.*ScriptAlias \/nagios\/cgi-bin /s|/nagios/cgi-bin .*|/nagios/cgi-bin '$NAGIOS_CGI_DIR'|' $NAGIOS_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#			sed -i -e '/^.*<Directory.*cgi"/s|<Directory ".*|<Directory "'$NAGIOS_CGI_DIR'">|' $NAGIOS_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#
#			sed -i -e '/^.*Alias \/nagios /s|/nagios .*|/nagios '$NAGIOS_THEME_DIR'|' $NAGIOS_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#			sed -i -e '/^.*<Directory.*nagios"/s|<Directory ".*|<Directory "'$NAGIOS_THEME_DIR'">|' $NAGIOS_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#
#			if [ $LOGGING -eq 1 ]; then grep "Alias " $NAGIOS_APACHE_CONF >&4; fi
#			if [ $LOGGING -eq 1 ]; then grep "<Directory" $NAGIOS_APACHE_CONF >&4; fi
#			if [ $LOGGING -eq 1 ]; then echo >&4; fi
#		fi
#	else
#		if [ -f $NAGIOS_APACHE_CONF ]
#		then
#			rm $NAGIOS_APACHE_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#		fi
	fi

	return $RETVAL

}

# ######### Reconfigure Apache2 default ssl function #########
#
reconfigure_apache_default_ssl(){

	local RETVAL=0

	if [ "$APACHE2_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Apache2 default ssl ----------------------------------------"
		echo

		if [ -f $APACHE2_CONF_DEFAULT_SSL ] && [ -d $ELMA_WWW_DIR ]
		then
			sed -i -e '/^.*ServerAdmin/s/ServerAdmin.*/ServerAdmin '$ELMA_MAIL_ADDRESS'/' $APACHE2_CONF_DEFAULT_SSL ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/^.*ServerName/s/ServerName.*/ServerName '$ELMA_WWW_FQDN'/' $APACHE2_CONF_DEFAULT_SSL ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/^.*DocumentRoot \//s|DocumentRoot .*|DocumentRoot '$ELMA_WWW_DIR'|' $APACHE2_CONF_DEFAULT_SSL ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			sed -i -e '/^.*<Directory "\/srv\/www\/htdocs/s|<Directory ".*|<Directory "'$ELMA_WWW_DIR'">|' $APACHE2_CONF_DEFAULT_SSL ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			if [ $LOGGING -eq 1 ]; then grep "ServerAdmin" $APACHE2_CONF_DEFAULT_SSL >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "ServerName" $APACHE2_CONF_DEFAULT_SSL >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "ProxyPass" $APACHE2_CONF_DEFAULT_SSL >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "DocumentRoot" $APACHE2_CONF_DEFAULT_SSL >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "Alias" $APACHE2_CONF_DEFAULT_SSL >&4; fi
			if [ $LOGGING -eq 1 ]; then grep "<Directory" $APACHE2_CONF_DEFAULT_SSL >&4; fi
			if [ $LOGGING -eq 1 ]; then echo >&4; fi
		fi
	fi

	return $RETVAL

}

# ######### Enable and reconfigure Apache2 service #########
#
reconfigure_apache_service(){

	local RETVAL=0

	if [ "$APACHE2_ENABLED" = "Yes" ]
	then
		echo "--- `date +%T.%N | cut -c 1-12` - Enable Apache2 service -------------------------------------------------"
		echo

		# Bugfix
		if [ -f $APACHE2_CONF_DIR/conf.d/elma-backup.conf ]
		then
			rm /etc/apache2/conf.d/elma-backup.conf
		fi

		# ######### Update Apache ServerName variable #########
		#
		ELMA_WWW_FQDN="$HOST_FQDN"
		sed -i -e '/ELMA_WWW_FQDN=""/s/=.*/=\x27'"$ELMA_WWW_FQDN"'\x27/' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		# ######### Update Apache ServerAdmin variable #########
		#
		if [ -z "$ELMA_MAIL_ADDRESS" ]
		then
			ELMA_MAIL_ADDRESS="webmaster@$HOST_FQDN"
			sed -i -e '/ELMA_MAIL_ADDRESS=""/s/=.*/=\x27'"$ELMA_MAIL_ADDRESS"'\x27/' $ELMA_CONF ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		fi

		systemctl is-enabled apache2.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -ne 0 ]
		then
			a2enmod proxy ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			a2enmod mod_proxy_balancer ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			a2enmod proxy_http ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			a2enmod rewrite ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#			a2enmod passenger ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			a2enmod ldap ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			a2enmod authnz_ldap ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			a2enmod auth_xradius ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			a2enmod status ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
#			a2enmod mod_socache_shmcb ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			a2enflag SSL ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

			systemctl enable apache2.service; if [ $? -ne 0 ]; then exit; fi
			echo
			RETVAL=0

			# ######### Generate self-signed apache2 certificate #########
			#
			if [ ! -f $APACHE2_CONF_DIR/ssl.crt/server.crt.elma ] || [ ! -f $APACHE2_CONF_DIR/ssl.key/server.key.elma ]
			then
				echo "--- `date +%T.%N | cut -c 1-12` - generate self-signed apache2 certificate -------------------------------"
				echo
				gensslcert ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				cp $APACHE2_CONF_DIR/ssl.crt/server.crt $APACHE2_CONF_DIR/ssl.crt/server.crt.elma ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				cp $APACHE2_CONF_DIR/ssl.key/server.key $APACHE2_CONF_DIR/ssl.key/server.key.elma ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
				echo
			fi
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Reconfigure Apache2 service --------------------------------------------"
		echo

		reconfigure_apache_redirect_http ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		reconfigure_apache_default_ssl ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		reconfigure_apache_default_authentication ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		reconfigure_apache_elasticsearch ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		reconfigure_apache_kibana3 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		reconfigure_apache_nagios ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		reconfigure_elma_web_directory ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi

		echo "--- `date +%T.%N | cut -c 1-12` - Restarting Apache2 service ---------------------------------------------"
		echo
		systemctl is-enabled apache2.service >/dev/null 2>&1 ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		echo >/dev/null 2>&1
		systemctl reload apache2.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
		systemctl status apache2.service
		echo
	else
		echo "--- `date +%T.%N | cut -c 1-12` - Disable Apache2 service ------------------------------------------------"
		echo
		systemctl is-enabled apache2.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl disable apache2.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			echo
		else
			RETVAL=0
		fi

		echo "--- `date +%T.%N | cut -c 1-12` - Stop Apache2 service ---------------------------------------------------"
		echo
		systemctl is-active apache2.service >/dev/null 2>&1 ; RETVAL=$?
		echo >/dev/null 2>&1
		if [ $RETVAL -eq 0 ]
		then
			systemctl stop apache2.service ; if [ $? -ne 0 ]; then error_exit "$LINENO"; fi
			systemctl status apache2.service
		else
			RETVAL=0
		fi
	fi

	return $RETVAL

}